Type BRUSH
	Field _material:SMaterial

	Method Delete()
		_material=Null
	EndMethod

    Method SetColor(red:Float,green:Float,blue:Float)
		If Not _material Or Not _material.isValid() Then RuntimeError "Material is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		Local colorf:SColorf=SColorf.createFromSColor(color)
		
		_material.getDiffuseColor().set(colorf.getAlpha(),colorf.getRed(),colorf.getGreen(),colorf.getBlue())
    	_material.getAmbientColor().set(colorf.getAlpha(),colorf.getRed(),colorf.getGreen(),colorf.getBlue())
    EndMethod

	Method setAmbientColor(red:Float,green:Float,blue:Float)
		If Not _material Or Not _material.isValid() Then RuntimeError "Material is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		Local colorf:SColorf=SColorf.createFromSColor(color)
		
    	_material.getAmbientColor().set(0,colorf.getRed(),colorf.getGreen(),colorf.getBlue())
	EndMethod

	Method setDiffuseColor(red:Float,green:Float,blue:Float)
		If Not _material Or Not _material.isValid() Then RuntimeError "Material is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		Local colorf:SColorf=SColorf.createFromSColor(color)
		
    	_material.getDiffuseColor().set(0,colorf.getRed(),colorf.getGreen(),colorf.getBlue())
	EndMethod
	
	Method setEmissiveColor(red:Float,green:Float,blue:Float)
		If Not _material Or Not _material.isValid() Then RuntimeError "Material is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		Local colorf:SColorf=SColorf.createFromSColor(color)
		
    	_material.getEmissiveColor().set(0,colorf.getRed(),colorf.getGreen(),colorf.getBlue())
	EndMethod

	Method setSpecularColor(red:Float,green:Float,blue:Float)
		If Not _material Or Not _material.isValid() Then RuntimeError "Material is invalid"
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		Local colorf:SColorf=SColorf.createFromSColor(color)
		
    	_material.getSpecularColor().set(0,colorf.getRed(),colorf.getGreen(),colorf.getBlue())
	EndMethod														

	Method setFX(EMF_MATERIAL_FLAG:Int,bNewVal:Int)
		If Not _material Or Not _material.isValid() Then RuntimeError "Material is invalid"
		_material.setFlag(EMF_MATERIAL_FLAG,bNewVal)
	EndMethod

	Method setShininess(shininess:Float=20.0,MaterialIDX:Int=0)
		If Not _material Or Not _material.isValid() Then RuntimeError "Material is invalid"
		_material.setShininess(shininess)
	EndMethod

	Method SetBlend(E_MATERIAL_TYPE:Int)
		If Not _material Or Not _material.isValid() Then RuntimeError "Material is invalid"
		_material.setMaterialType(E_MATERIAL_TYPE)
	EndMethod

	' TODO: note index only support 0,1
	' TODO: implement animated texture support
	Method setTexture(texture:TEXTURE,frame:Int=0,index:Int=0)
		If Not _material Or Not _material.isValid() Then RuntimeError "Material is invalid"
		_material.setTexture(index,texture.tex)
	EndMethod
	
EndType

Rem
bbdoc: Creates a brush
about: <i>Parameters</i>
<ul>
<li>entity:ib3d_Entity - entity instance</li>
</ul>
EndRem
Function ib3d_CreateBrush:BRUSH(red:Float=255,green:Float=255,blue:Float=255)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.createBrush(red,green,blue)
EndFunction

' TODO: note/figure out parameters for texture and scale
Function ib3d_LoadBrush:BRUSH(texture_file:String)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.loadBrush(texture_file)
EndFunction

' TODO: note new param
Function ib3d_BrushColor(brush:BRUSH,red:Float,green:Float,blue:Float)
	If Not brush Then Throw "Brush does not exist" 
	brush.setColor(red,green,blue)
EndFunction

' TODO: NOTE NEW
Function ib3d_BrushAmbientColor(brush:BRUSH,red:Float,green:Float,blue:Float)
	If Not brush Then Throw "Brush does not exist" 
	brush.setAmbientColor(red,green,blue)
EndFunction

' TODO: NOTE NEW
Function ib3d_BrushDiffuseColor(brush:BRUSH,red:Float,green:Float,blue:Float)
	If Not brush Then Throw "Brush does not exist" 
	brush.setDiffuseColor(red,green,blue)
EndFunction

' TODO: NOTE NEW
Function ib3d_BrushSpecularColor(brush:BRUSH,red:Float,green:Float,blue:Float)
	If Not brush Then Throw "Brush does not exist" 
	brush.setSpecularColor(red,green,blue)
EndFunction

' TODO: NOTE NEW
Function ib3d_BrushEmissiveColor(brush:BRUSH,red:Float,green:Float,blue:Float)
	If Not brush Then Throw "Brush does not exist" 
	brush.setEmissiveColor(red,green,blue)
EndFunction

Function ib3d_FreeBrush(brush:BRUSH Var)
	brush=Null	
EndFunction

' TODO: note that second param is different
Function ib3d_BrushShininess(brush:BRUSH,shininess:Float=20.0)
	If Not brush Then Throw "Brush does not exist" 
	brush.setShininess(shininess)
EndFunction

' TODO: new params
Function ib3d_BrushFX(brush:BRUSH,EMF_MATERIAL_FLAG:Int,bNewVal:Int)
	If Not brush Then Throw "Brush does not exist" 
	brush.setFX(EMF_MATERIAL_FLAG,bNewVal)
EndFunction

' TODO: new params
Function ib3d_BrushBlend(brush:BRUSH,E_MATERIAL_TYPE:Int=EMT_SOLID)
	If Not brush Then Throw "Brush does not exist" 
	brush.setBlend(E_MATERIAL_TYPE)
EndFunction

' TODO: note index only support 0,1
' TODO: implement animated texture support
Function ib3d_BrushTexture(brush:BRUSH,texture:TEXTURE,frame:Int=0,index:Int=0)
	If Not brush Then Throw "Brush does not exist" 
	brush.setTexture(texture,frame,index)	
EndFunction

Rem
LoadBrush	- combination of SMaterial and IVideoDriver.addTexture()

  LoadBrush ( file$[,flags][,u_scale][,v_scale]  

Parameters:


file$ - filename
flags - brush flags

flags (optional) - flags can be added to combine effects:
1: Color
2: Alpha
4: Masked
8: Mipmapped
16: Clamp U
32: Clamp V
64: Spherical reflection map

u_scale - brush u_scale
v_scale - brush v_scale 


BrushAlpha - SMaterial.colors()
EndRem
