Global _g_ib3d_engine:ib3d_engine=Null

' create an initial instance of the engine so all functions work out of the box
New ib3d_engine

' TODO: change renderentities to render the GUI as well
' TODO: change renderentities to render individual cameras

Rem
TODO: add capabilities to queue primitives to render such as triangles and lines
like:
Local material:SMaterial=SMaterial.create()
'material.setTexture1()
'material.setLighting(False)
_g_ib3d_engine.driver.SetTransform(ETS_WORLD,Matrix4.create())
_g_ib3d_engine.driver.draw3dLine(ib3d_LastPickRay().getStart(),ib3d_LastPickRay().getEnd(),_SCOLOR(0,255,0,0))
_g_ib3d_engine.driver.draw3dTriangle(_g_ib3d_pick_manager.getLastPickedTriangle(),_SCOLOR(0,255,0,0))
EndRem

Rem
Graphics3D (done)
Dither	
WBuffer	
AntiAlias (supported through creation params)
Wireframe	
HWMultiTex	
AmbientLight (done)	
ClearCollisions	
Collisions	
UpdateWorld	
CaptureWorld	
RenderWorld (done except for tween)	
ClearWorld	
LoaderMatrix
EndRem

Type ib3d_engine

	Field screenRect:Recti=Null
	Field clsColor:SColor=Null
	Field bClsBackBuffer:Int=True
	Field bClsZBuffer:Int=True
	Field device:IrrlichtDevice=Null
	Field smgr:ISceneManager=Null
	Field driver:IVideoDriver=Null
	Field guienv:IGUIEnvironment=Null
	Field recv:ib3d_EventReceiver=Null
	Field _speed:Float=1.0
	Field bRendering:Int=False ' True if currently rendering
	Field lastTForm:Vector3df=Null
		
	Method New()
		_g_ib3d_engine=Self
	EndMethod

	Method SetEventReceiver(receiver:IEventReceiver = Null)
		If Not receiver Then device.setEventReceiver(recv) Else device.setEventReceiver(receiver)
	EndMethod

	Method Graphics3DParams:Int(params:SIrrlichtCreationParameters)
		
		device=IrrlichtDevice.createFromParams(params)

		InitEngine()
										
		Return (device<>Null)			
	EndMethod
	
	Method Graphics3D:Int(width:Int,height:Int,depth:Int=16,mode:Int=0,stencilBufferShadows:Int=True,vsync:Int=False,deviceType:Int=EDT_OPENGL)
		Local fullscreen:Int=True
	
		Select mode
			Case 0
				fullscreen=True

				' in debug we are windowed
				?Debug
					fullscreen=False
				?
			Case 1
				fullscreen=True
			Case 2
				fullscreen=False
			Case 3
				fullscreen=False			
		EndSelect
	
		device=IrrlichtDevice.Create(deviceType,Dimension2di.Create(width,height),depth,fullscreen,stencilBufferShadows,vsync,Null)
		
		InitEngine()
						
		Return (device<>Null)
	EndMethod

	Method InitEngine()
		' get references to the components of Irrlicht
		If device Then
			driver=device.getVideoDriver()
			smgr=device.getSceneManager()		
			guienv=device.getGUIEnvironment()		
			recv=ib3d_EventReceiver(IEventReceiver.Create(ib3d_EventReceiver.generate))
			SetEventReceiver()
		EndIf
		If driver Then 
			screenRect=Recti.createFromVals(0,0,driver.getScreenSize().getWidth(),driver.getScreenSize().getHeight())
			setAmbientLight() ' init the ambient light
		EndIf
		If device Then SetClsColor()
	EndMethod

	Method OnEvent:Int(event:SEvent)
		Return False
	EndMethod
	
	Method EndGraphics()
		If Not driver Or Not driver.IsValid() Then Throw "Graphics have not yet been initialized"
		
		' TODO: loop through all lists and destroy
		
		device.CloseDevice()
		device.drop()		
		recv=Null
		screenRect=Null
		clsColor=Null
		guienv=Null
		smgr=Null
		driver=Null
		device=Null
	EndMethod

	Method setAmbientLight(red:Int=127,green:Int=127,blue:Int=127)
		If Not driver Or Not driver.IsValid() Then Throw "Graphics have not yet been initialized"		
    	driver.setAmbientLight(SColorf.createFromSColor(SColor.createFromVals(0,red,green,blue)))
	EndMethod

	' initialize the scene render
	Method RenderWorldBegin:Int()
		If Not driver Or Not driver.IsValid() Then Throw "Graphics have not yet been initialized"
		
		' make sure the window is active
		'If (device.run() And device.isWindowActive())
		If device.run()
			bRendering=True
		EndIf
		
		Return bRendering
	EndMethod
	
	' renders the entities to each camera
	Method RenderEntities:Int()
		Local bSuccess:Int=False
		
		' make sure we are rendering
		If bRendering
			' set the initial viewport to fullscreen
			driver.SetViewport(screenRect)
			
			' start the scene			
			driver.beginScene(bClsBackBuffer,bClsZBuffer,clsColor)
			
			If CAMERA.CameraList<>Null And smgr And smgr.IsValid()
				For Local camera:CAMERA=EachIn CAMERA.CameraList 
					If camera._node.IsVisible()
						camera.updateCam() ' update where the camera is looking 
						' apply any fog settings for the camera
						camera.applyFog()
						smgr.setActiveCamera(camera._cam)
						driver.SetViewport(camera._viewport)

						' update sprites based on this cam
						SPRITE.update(camera)
												
						smgr.drawAll()
					EndIf
				Next
				bSuccess=True
			EndIf
		EndIf
		
		Return bSuccess
	EndMethod
	
	' renders the GUI
	Method RenderGUI:Int()
		Local bSuccess:Int=False
		
		' make sure we are rendering
		If bRendering
			If guienv And guienv.IsValid()
				guienv.drawAll()
				bSuccess=True
			EndIf
		EndIf
		
		Return bSuccess
	EndMethod

	Method RenderDrawCmds:Int()
		Local bSuccess:Int=False
		
		' make sure we are rendering
		If bRendering
			If driver And driver.IsValid()
				_g_ib3d_draw_manager.Draw()
				bSuccess=True
			EndIf
		EndIf
		
		Return bSuccess
	EndMethod
	
	Method RenderWorldEnd()
		If bRendering
			driver.endScene()
			Delta.Update()
		EndIf
		bRendering=False
	EndMethod
										
	' TODO: NOTE showGUI parameter
	Method RenderWorld(showGUI:Int=True,tween:Float=1)
		If RenderWorldBegin()
			If RenderEntities() And RenderDrawCmds()
				If showGUI Then RenderGUI()
			EndIf
		EndIf
		RenderWorldEnd()
	EndMethod
	
	Method UpdateWorld(anim_speed:Float=1.0)
		_g_ib3d_collision_manager.detectCollisions()
	EndMethod
	
	Method setSpeed(new_speed:Float=1.0)
		_speed=new_speed
	EndMethod
	
	Method getSpeed:Float()
		Return _speed
	EndMethod

	Method TFormPoint:Vector3df(x:Float,y:Float,z:Float,source:ib3d_Entity,dest:ib3d_Entity)
		If source And (Not source._node Or Not source._node.isValid()) Then RuntimeError "Source node is invalid"
		If dest And (Not dest._node Or Not dest._node.isValid()) Then RuntimeError "Destination node is invalid"

		lastTForm=_VECTOR3DF(x,y,z)
	
		' first convert to world from the source
		If source Then source._node.getAbsoluteTransformation().transformVect(lastTForm)
	
		' now from world to local
		If dest Then dest._node.getRelativeTransformation().transformVect(lastTForm)
		
		Return lastTForm
	EndMethod

	Method TFormedX:Float()
		If lastTForm Then Return lastTForm.getX() Else RuntimeError "A TForm function has not been performed"
	EndMethod

	Method TFormedY:Float()
		If lastTForm Then Return lastTForm.getY() Else RuntimeError "A TForm function has not been performed"
	EndMethod
	
	Method TFormedZ:Float()
		If lastTForm Then Return lastTForm.getZ() Else RuntimeError "A TForm function has not been performed"
	EndMethod

	Method LastTFormed(x:Float Var,y:Float Var,z:Float Var)
		If Not lastTForm Then RuntimeError "A TForm function has not been performed"
		x=lastTForm.getX()
		y=lastTForm.getY()
		z=lastTForm.getZ()
	EndMethod
	
	' ----------------------------------------------------------
	' CAMERA
	' ----------------------------------------------------------

	Method generateCamera:CAMERA()
		Return New CAMERA
	EndMethod
	
	Method CreateCamera:CAMERA(parent:ib3d_Entity=Null,camType:Int=CT_NORMAL)
		Local retval:CAMERA=generateCamera()
		retval._type=camType
				
		If camType=CT_FPS
			retval.setNode(smgr.addCameraSceneNodeFPS())
		ElseIf camType=CT_MAYA
			retval.setNode(smgr.addCameraSceneNodeMaya())
		ElseIf camType=CT_NORMAL
			'retval.setNode(smgr.addCameraSceneNodeFPS())
			'retval._cam.setInputReceiverEnabled(False)
			retval.setNode(smgr.addCameraSceneNode())
		Else
			'retval.setNode(smgr.addCameraSceneNodeFPS())
			'retval._cam.setInputReceiverEnabled(False)
			retval.setNode(smgr.addCameraSceneNode())		
		EndIf
		
		' set the parent if passed
		If parent Then retval._node.setParent(parent._node)
		
		' default to full window viewport
		retval.setViewportToScreen()
		' default near/far
		retval.setRange()

		Return retval
	EndMethod
	
	Method SetClsColor(r:Int=0,g:Int=0,b:Int=0)
		clsColor=SColor.createFromVals(0,r,g,b)
	EndMethod
	
	Method setClsMode(bBackBuffer:Int=True,bZBuffer:Int=True)
		bClsBackBuffer=bBackBuffer
		bClsZBuffer=bZBuffer
	EndMethod

	' ----------------------------------------------------------
	' LIGHT
	' ----------------------------------------------------------	

	Method generateLight:LIGHT()
		Return New LIGHT
	EndMethod
	
	' TODO: NOTE lack of parameter
	Method CreateLight:LIGHT(parent:ib3d_Entity=Null)
		Local retval:LIGHT=generateLight()
		
		retval.setNode(smgr.addLightSceneNode())
		' set the parent if passed
		If parent Then retval._node.setParent(parent._node)
		retval.setRange()
		retval.SetColor(255,255,255)
		Return retval
	EndMethod

	' ----------------------------------------------------------
	' TEXTURE
	' ----------------------------------------------------------

	Method generateTexture:TEXTURE()
		Return New TEXTURE
	EndMethod
	
	Method LoadTexture:TEXTURE(file:String)
		Local retval:TEXTURE=generateTexture()
		
		retval.tex=driver.getTexture(file)
		Return retval		
	EndMethod

	Method CreateTexture:TEXTURE()

		Local retval:TEXTURE=generateTexture()
		Rem
		driver.createRenderTargetTexture(_DIMENSION2DI(256,256))

		If surface_type=SF_LIGHTMAP
			retval._mbuf=SMeshBufferLightMap.create()
		ElseIf surface_type=SF_TANGENTS
			retval._mbuf=SMeshBufferTangents.create()
		Else
			retval._mbuf=SMeshBuffer.create()
		EndIf
		
		' add this new buffer to the passed mesh
		SMesh(mesh._mesh).addMeshBuffer(retval._mbuf)
		EndRem
		Return retval
	EndMethod
	
	' ----------------------------------------------------------
	' MESH
	' ----------------------------------------------------------

	Method generateMesh:MESH()
		Return New MESH
	EndMethod
	
	Method LoadAnimMesh:MESH(file:String,parent:ib3d_Entity=Null)
		Local retval:MESH=generateMesh()
		retval._mesh=smgr.getMesh(file)
		If Not retval._mesh Or Not retval._mesh.IsValid() Then RuntimeError "Unable to load mesh: "+file
		
		retval.setNode(smgr.addAnimatedMeshSceneNode(IAnimatedMesh(retval._mesh)))
		
		If parent Then retval._node.setParent(parent._node)
		
		Return retval		
	EndMethod
				
	Method LoadMesh:MESH(file:String,parent:ib3d_Entity=Null)
		Local retval:MESH=generateMesh()
		retval._mesh=smgr.getMesh(file)
		If Not retval._mesh Or Not retval._mesh.IsValid() Then RuntimeError "Unable to load mesh: "+file
		
		retval.setNode(smgr.addMeshSceneNode(IAnimatedMesh(retval._mesh).getMesh(0)))
		
		If parent Then retval._node.setParent(parent._node)
		
		Return retval		
	EndMethod

	Method loadOctTreeMesh:MESH(file:String,parent:ib3d_Entity=Null)
		Local retval:MESH=generateMesh()
		retval._mesh=smgr.getMesh(file)
		If Not retval._mesh Or Not retval._mesh.IsValid() Then RuntimeError "Unable to load mesh: "+file
		
		retval.setNode(smgr.addOctTreeSceneNode(IAnimatedMesh(retval._mesh).getMesh(0)))
		
		If parent Then retval._node.setParent(parent._node)
		
		Return retval		
	EndMethod

	Method CreateMesh:MESH(parent:ib3d_Entity=Null)
		Local retval:MESH=generateMesh()
		retval._mesh=SMesh.Create()
		If Not retval._mesh Or Not retval._mesh.IsValid() Then RuntimeError "Unable to create mesh"
		
		retval.setNode(smgr.addMeshSceneNode(SMesh(retval._mesh)))
		
		If parent Then retval._node.setParent(parent._node)  ' set the parent if there is one
		
		Return retval
	EndMethod

	Method generateSurface:SURFACE()
		Return New SURFACE
	EndMethod
			
	Method CreateSurface:SURFACE(mesh:MESH,surface_type:Int=SF_NORMAL)
		If Not mesh._mesh Or Not mesh._mesh.isValid() Then RuntimeError "Mesh is invalid"
		If Not SMesh(mesh._mesh) Then RuntimeError "Mesh is not editable"
		Local retval:SURFACE=generateSurface()
		If surface_type=SF_LIGHTMAP
			retval._mbuf=SMeshBufferLightMap.Create()
		ElseIf surface_type=SF_TANGENTS
			retval._mbuf=SMeshBufferTangents.Create()
		Else
			retval._mbuf=SMeshBuffer.Create()
		EndIf
		
		' add this new buffer to the passed mesh
		SMesh(mesh._mesh).addMeshBuffer(retval._mbuf)
		
		Return retval
	EndMethod
	
	' ----------------------------------------------------------
	' PRIMITIVES
	' ----------------------------------------------------------

	Method CreateCube:PRIMITIVE(parent:ib3d_Entity=Null)
		Local retval:PRIMITIVE=generatePrimitive()
		
		retval.setNode(smgr.addCubeSceneNode())
		
		If parent Then retval._node.setParent(parent._node)
		
		Return retval		
	EndMethod

	Method CreateSphere:PRIMITIVE(segments:Int=8,parent:ib3d_Entity=Null)
		Local retval:PRIMITIVE=generatePrimitive()
		
		retval.setNode(smgr.addSphereSceneNode(1.0,segments))
		
		If parent Then retval._node.setParent(parent._node)
		
		Return retval		
	EndMethod

	Method generatePrimitive:PRIMITIVE()
		Return New PRIMITIVE
	EndMethod

	' ----------------------------------------------------------
	' SPRITE
	' ----------------------------------------------------------

    Method LoadSprite:SPRITE(tex_file:String,tex_flag:Int=EMT_SOLID,rotate_mode:Int=1,parent_ent:ib3d_Entity=Null)

        Local NewSprite:Sprite=CreateSprite(rotate_mode,parent_ent)
        
        ib3d_EntityBlend(newsprite,tex_flag)
        NewSprite.tex = ib3d_LoadTexture(tex_file)
        ib3d_EntityTexture(newsprite,newsprite.tex)
        
        Return NewSprite        
    EndMethod

    Method CreateSprite:SPRITE(rotate_mode:Int=1,parent_ent:ib3d_Entity=Null)

        Local NewSprite:SPRITE=generateSprite()
        newsprite.cont:+1
        If rotate_mode=1  'uses the already created spmesh, which is faster, if rotate all sprites with spmesh rotates.
            If SPRITE.spmesh = Null 
                SPRITE.spmesh = smgr.addHillPlaneMesh("sprite"+SPRITE.cont, _dimension2df(1.0 , 1.0) , _dimension2di(1 , 1) )
                If Not SPRITE.spmesh Then RuntimeError "Unable to create sprite mesh"
                NewSprite._mesh=SPRITE.spmesh
                NewSprite.setNode(smgr.addMeshSceneNode(IAnimatedMesh(newsprite._mesh).getMesh(0)))
                _RotateAnimMesh(IAnimatedMesh(SPRITE.spmesh).handle,newsprite._node.handle,-90,0,0) 'HillPlaneMesh needs to be rotated to face the camera
            Else
	            NewSprite._mesh=SPRITE.spmesh
	            If Not NewSprite._mesh Then RuntimeError "Unable to create sprite mesh"    
	            NewSprite.setNode(smgr.addMeshSceneNode(IAnimatedMesh(newsprite._mesh).getMesh(0)))
            End If
        Else 'create a new mesh, slower but allows rotation
            NewSprite._mesh= smgr.addHillPlaneMesh("sprite"+newSprite.cont, _dimension2df(1.0 , 1.0) , _dimension2di(1 , 1) )
            If Not NewSprite._mesh Then RuntimeError "Unable to create sprite mesh"    
            NewSprite.setNode(smgr.addMeshSceneNode(IAnimatedMesh(newsprite._mesh).getMesh(0)))
            _RotateAnimMesh(IAnimatedMesh(SPRITE.spmesh).handle,newsprite._node.handle,-90,0,0) 'HillPlaneMesh needs to be rotated to face the camera
        End If
        If parent_ent Then NewSprite._node.setParent(parent_ent._node)  ' set the parent if there is one
        newsprite.setFX(EMF_LIGHTING,False) 
        Return NewSprite
    EndMethod

	Method generateSprite:SPRITE()
		Return New SPRITE
	EndMethod
					
	' ----------------------------------------------------------
	' INPUT
	' ----------------------------------------------------------

	Method KeyHit:Int(scancode:Int,bClear:Int=True)
		If Not recv Then Return 0
		Return recv.KeyHitCnt(scancode,bClear)
	EndMethod
				
	Method KeyDown:Int(scancode:Int)
		If recv Then Return recv.IsKeyDown(scancode) Else Return False
	EndMethod

   	Method setShadowColor(alpha:Int=150,red:Int=0,green:Int=0,blue:Int=0)
   		smgr.setShadowColor(SColor.createFromVals(alpha,red,green,blue))
   	EndMethod

	Method MouseX:Int()
		Return device.getCursorControl().getPosition().getX()
	EndMethod
	
	Method MouseY:Int()
		Return device.getCursorControl().getPosition().getY()
	EndMethod

Rem
	Method AddGUIEventHandler(element_id:Int, handler:ib3d_GUIEventHandler)
		recv.AddGUIEventHandler(element_id, handler)
	EndMethod

	Method RemoveGUIEventHandler(element_id:Int)
		recv.RemoveGUIEventHandler(element_id)
	EndMethod
EndRem

	' ----------------------------------------------------------
	' PIVOT
	' ----------------------------------------------------------

	Method generatePivot:PIVOT()
		Return New PIVOT
	EndMethod
	
	Method CreatePivot:PIVOT(parent:ib3d_Entity=Null)
		Local retval:PIVOT=generatePivot()
		retval.setNode(smgr.addEmptySceneNode())
		
		If parent Then retval._node.setParent(parent._node)
		
		Return retval		
	EndMethod
		
	' ----------------------------------------------------------
	' PLANE
	' ----------------------------------------------------------

	Method generatePlane:PLANE()
		Return New PLANE
	EndMethod
	
	Method createPlane:PLANE(tileSizeX:Float,tileSizeY:Float,tileCountX:Int,tileCountY:Int,parent:ib3d_Entity=Null)
		Local retval:PLANE=generatePlane()
		
		retval._mesh=smgr.addHillPlaneMesh("hillplanemesh_"+PLANE.PlaneList.Count(),Dimension2df.Create(tileSizeX,tileSizeY),Dimension2di.Create(tileCountX,tileCountY))
		If Not retval._mesh Then RuntimeError "Unable to create HillPlaneMesh"
		retval.setNode(smgr.addAnimatedMeshSceneNode(IAnimatedMesh(retval._mesh)))

		If parent Then retval._node.setParent(parent._node)

		Return retval		
	EndMethod

	Method addZipFileArchive:Int(file:String,bIgnoreCase:Int=True,bIgnorePaths:Int=True)
		device.getFileSystem().addZipFileArchive(file,bIgnoreCase,bIgnorePaths)
	EndMethod

	' ----------------------------------------------------------
	' TERRAIN
	' ----------------------------------------------------------

	Method generateTerrain:TERRAIN()
		Return New TERRAIN
	EndMethod

	Method loadTerrain:TERRAIN(file:String,parent:ib3d_Entity=Null)
		Local retval:TERRAIN=generateTerrain()
		
		retval.setNode(smgr.addTerrainSceneNodeFromFile(file))
		
		If parent Then retval._node.setParent(parent._node)
		
		Return retval		
	EndMethod

	' ----------------------------------------------------------
	' BRUSH
	' ----------------------------------------------------------

	Method generateBrush:BRUSH()
		Return New BRUSH
	EndMethod			
	
	Method CreateBrush:BRUSH(red:Float=255,green:Float=255,blue:Float=255)
		Local retval:BRUSH=generateBrush()
		Local color:SColor=SColor.createFromVals(0,red,green,blue)
		Local colorf:SColorf=SColorf.createFromSColor(color)
		
		retval._material=SMaterial.Create()
		retval._material.getDiffuseColor().set(colorf.GetAlpha(),colorf.getRed(),colorf.getGreen(),colorf.getBlue())
    		retval._material.getAmbientColor().set(colorf.GetAlpha(),colorf.getRed(),colorf.getGreen(),colorf.getBlue())
		
		Return retval
	EndMethod

	Method LoadBrush:BRUSH(texture_file:String)
		Local retval:BRUSH=CreateBrush()
		ib3d_BrushTexture(retval,ib3d_LoadTexture(texture_file))
		Return retval
	EndMethod


	' ----------------------------------------------------------
	' ----------------------------------------------------------
	' GUI
	' ----------------------------------------------------------

	' ----------------------------------------------------------
	' STATIC TEXT
	' ----------------------------------------------------------		
	
	Rem
	Method generateGUIText:GUI_TEXT()
		Return New GUI_TEXT
	EndMethod
	
	Method CreateGUIText(text:String, x:Int, y:Int)
		Local retval:TERRAIN=generateTerrain()

		Local element:IGUIStaticText = guienv.addStaticText(text:String,_RECTI(,bBorder:Int=False,bWordWrap:Int=True,parent:IGUIElement=Null,id:Int=-1,bFillBackground:Int=False)		
		retval.setNode(smgr.addTerrainSceneNodeFromFile(file))
		
		If parent Then retval._node.setParent(parent._node)
		
		Return retval		
		
	EndMethod
	EndRem
EndType

Rem
bbdoc_removethis: <description>
returns: <what it returns>
about: <i>Parameters</i>
<ul>
<li> - </li>
</ul>
<details>
keyword: <keyword>
EndRem

Rem
bbdoc: Sets 3D Graphics mode and initializes the graphics system
returns: True if success
about: <i>Parameters</i>
<ul>
<li>width:Int - Width of screen resolution.</li>
<li>height:Int - Height of screen resolution.</li>
<li>depth:Int - Bits per pixel in fullscreen mode.  Ignored if windowed mode.  Defaults to 16.</li>
<li>mode:Int - Mode of display.  Defaults to 0.<br>
0: windowed (if possible) in debug mode, fullscreen in non-debug mode<br>
1: fullscreen always<br>
2. windowed always<br>
3: windowed/scaled always<br></li>
<li>stencilBufferShadows:Int - Specifies if the stencil buffer should be enabled.  Set this to true, if you want the engine be able to draw stencil buffer shadows. Note that not all devices are able to use the stencil buffer. If they don't no shadows will be drawn.  Defaults to True.</li>
<li>vsync:Int - Specifies vertical syncronisation: If set to true, the driver will wait for the vertical retrace period, otherwise not.  Defaults to False.</li>
<li>deviceType:Int - Type of the device.  Defaults to EDT_OPENGL.  Available Types: EDT_OPENGL, EDT_BURNINGVIDEO, EDT_DIRECT3D9, EDT_DIRECT3D8.</li>
</ul>
EndRem
Function ib3d_Graphics3D:Int(width:Int,height:Int,depth:Int=16,mode:Int=0,stencilBufferShadows:Int=True,vsync:Int=False,deviceType:Int=EDT_OPENGL)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.Graphics3D(width,height,depth,mode,stencilBufferShadows,vsync,deviceType)
EndFunction

Rem
bbdoc: Sets 3D Graphics mode and initializes the graphics system from a creation parameters object
returns: True if success
about: <i>Parameters</i>
<ul>
<li>params:SIrrlichtCreationParameters - parameters to use for the creation.</li>
</ul>
EndRem
Function ib3d_Graphics3DParams:Int(params:SIrrlichtCreationParameters)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.Graphics3DParams(params)
EndFunction

Rem
bbdoc: Uninitializes the graphics system.
about: Destroys all windows and objects since the ib3d_Graphics3D() call
EndRem
Function ib3d_EndGraphics()
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	_g_ib3d_engine.EndGraphics()
EndFunction

' TODO: tween param
Rem
bbdoc: Renders the current scene to the BackBuffer onto the rectangle defined by each cameras CameraViewport( ). Every camera not hidden by HideEntity( ) or with a CameraProjMode( ) of 0 is rendered.
about: <i>Parameters</i>
<ul>
<li> showGUI:Int - Set to False to not render GUI elements.  Defaults to True.</li>
<li> tween:Float - Currently not available.</li>
</ul>
EndRem
Function ib3d_RenderWorld(showGUI:Int=True,tween:Float=1)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	_g_ib3d_engine.RenderWorld(showGUI,tween)
EndFunction

Rem
bbdoc: Initializes rendering for the current frame
returns: True if the rendering for the frame was successfully intitialized
about: Used instead of ib3d_RenderWorld() if you need more control over the rendering process (ie custom rendering after entities are rendered).
EndRem
Function ib3d_RenderWorldBegin:Int()
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.RenderWorldBegin()
EndFunction

Rem
bbdoc: Renders all entities for the current frame
returns: True if entities were successfully rendered
about: Must be called between ib3d_RenderWorldBegin() and ib3d_RenderWorldEnd() calls.
EndRem
Function ib3d_RenderEntities:Int()
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.RenderEntities()
EndFunction

Rem
bbdoc: Renders all draw commands for the current frame
returns: True if draw commands were successfully rendered
about: Must be called between ib3d_RenderWorldBegin() and ib3d_RenderWorldEnd() calls and after ib3d_RenderEntities().
EndRem
Function ib3d_RenderDrawCmds:Int()
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.RenderDrawCmds()
EndFunction

Rem
bbdoc: Renders all GUI elements for the current frame
returns: True if GUI elements were successfully rendered
about: Must be called between ib3d_RenderWorldBegin() and ib3d_RenderWorldEnd() calls.
EndRem
Function ib3d_RenderGUI:Int()
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.RenderGUI()
EndFunction

Rem
bbdoc: Stops rendering of the current frame
about: This should only be used if ib3d_RenderWorldBegin() has been called.
EndRem
Function ib3d_RenderWorldEnd()
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	_g_ib3d_engine.RenderWorldEnd()
EndFunction

' TODO: hook into anim speed.  default anim speed is 100.
Rem
bbdoc: Animates all entities in the world, and performs collision checking. 
about: <i>Parameters</i>
<ul>
<li> anim_speed:Float - Allows you affect the animation speed of all entities at once. A value of 1 will animate entities at their usual animation speed, a value of 2 will animate entities at double their animation speed, and so on.  Defaults to 1.0.</li>
</ul>
NOTE: anim_speed is not yet available.
EndRem
Function ib3d_UpdateWorld(anim_speed:Float=1.0)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	_g_ib3d_engine.UpdateWorld(anim_speed)
EndFunction

Rem
bbdoc: Used to detect if a key is being held down. 
returns: False if the key is not held down, True if the key is held down.
about: <i>Parameters</i>
<ul>
<li>scancode:Int - corresponding key scancode</li>
</ul>
EndRem
Function ib3d_KeyDown:Int(scancode:Int)
	Return _g_ib3d_engine.KeyDown(scancode)
EndFunction

Rem
bbdoc: Returns the number of times a specified key has been hit since the last time you called the KeyHit() command.
returns: Times key was hit
about: <i>Parameters</i>
<ul>
<li>scancode:Int - corresponding key scancode</li>
</ul>
EndRem
Function ib3d_KeyHit:Int(scancode:Int)
	Return _g_ib3d_engine.KeyHit(scancode)
EndFunction

Rem
bbdoc: Returns the X location of the mouse on the screen
returns: X location
EndRem
Function ib3d_MouseX:Int()
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.MouseX()
EndFunction

Rem
bbdoc: Returns the Y location of the mouse on the screen
returns: Y location
EndRem
Function ib3d_MouseY:Int()
	Return _g_ib3d_engine.MouseY()
EndFunction

' TODO: needs to somehow adjust animspeed on all entities
' NOTE: if store animspeed when set on entity, then can adjust on the fly
' NOTE: default animspeed is 100 framespersecond
Rem
bbdoc: Sets the speed at which the iB3D engine runs
about: <i>Parameters</i>
<ul>
<li>new_speed:Float - new speed in % at which the engine runs.  Default is 1.0.</li>
</ul>
Turn, Move, and animation speeds are dynamically adjusted based on this value.
EndRem
Function ib3d_SetSpeed(new_speed:Float=1.0)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	_g_ib3d_engine.setSpeed(new_speed)
EndFunction

Rem
bbdoc: Returns the current speed setting of the iB3D engine
returns: The current engine speed in %
EndRem
Function ib3d_GetSpeed:Float()
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.getSpeed()
EndFunction

Rem
bbdoc: Sets the color of stencil buffers shadows
about: <i>Parameters</i>
<ul>
<li>alpha:Int - alpha component of shadow.  Defaults to 150.</li>
<li>red:Int - red component of shadow.  Defaults to 0.</li>
<li>green:Int - green component of shadow.  Defaults to 0.</li>
<li>blue:Int - blue component of shadow.  Defaults to 0.</li>
</ul>
EndRem
Function ib3d_ShadowColor(alpha:Int=150,red:Int=0,green:Int=0,blue:Int=0)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	_g_ib3d_engine.setShadowColor(alpha,red,green,blue)
EndFunction

Rem
bbdoc: Sets the ambient lighting color
about: <i>Parameters</i>
<ul>
<li>red:Int - red ambient light value</li>
<li>green:Int - green ambient light value</li>
<li>blue:Int - blue ambient light value</li>
</ul>
The green, red and blue values should be in the range 0-255. The default ambient light colour is 127,127,127.<br><br>
Ambient light is a light source that affects all points on a 3D object equally. So with ambient light only, all 3D objects will appear flat, as there will be no shading.<br><br>
Ambient light is useful for providing a certain level of light, before adding other lights to provide a realistic lighting effect.<br><br>
An ambient light level of 0,0,0 will result in no ambient light being displayed.
EndRem
Function ib3d_AmbientLight(red:Int=127,green:Int=127,blue:Int=127)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
   	_g_ib3d_engine.setAmbientLight(red,green,blue)
EndFunction

Rem
bbdoc: Adds an zip archive to the file system
returns: Returns true if the archive was added successful, false if not.
about: <i>Parameters</i>
<ul>
<li>file:String - Filename of the zip archive to add to the file system.</li>
<li>bIgnoreCase:Int - If set to true, files in the archive can be accessed without writing all letters in the right case.  Defaults to True.</li>
<li>bIgnorePaths:Int - If set to true, files in the added archive can be accessed without its complete path.  Defaults to True.</li>
</ul>
After calling this, the Irrlicht Engine will search and open files directly from this archive too. This is useful for hiding data from the end user, speeding up file access and making it possible to access for example Quake3 .pk3 files, which are nothing different than .zip files. 
EndRem
Function ib3d_AddZipFileArchive:Int(file:String,bIgnoreCase:Int=True,bIgnorePaths:Int=True)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.addZipFileArchive(file,bIgnoreCase,bIgnorePaths)
EndFunction

Rem
bbdoc: Transforms between coordinate systems
returns: Returns the transformed point as a vector3df
about: <i>Parameters</i>
<ul>
<li>x:Float - X coordinate of a point in 3d space </li>
<li>y:Float - Y coordinate of a point in 3d space </li>
<li>z:Float - Z coordinate of a point in 3d space </li>
<li>source:ib3d_Entity - instance of source entity, or Null for 3d world</li>
<li>dest:ib3d_Entity - instance of destination entity, or Null for 3d world</li>
</ul>
After using TFormPoint the new coordinates can be read with ib3d_TFormedX(), ib3d_TFormedY() and ib3d_TFormedZ(). <br><br>
See ib3d_EntityX() for details about local coordinates. <br><br>
Consider a sphere built with CreateSphere(). The 'north pole' is at (0,1,0). At first, local and global coordinates are the same. As the sphere is moved, turned and scaled the global coordinates of the point change. <br><br>
But it is always at (0,1,0) in the sphere's local space. 
EndRem
Function ib3d_TFormPoint:Vector3df(x:Float,y:Float,z:Float,source:ib3d_Entity,dest:ib3d_Entity)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.TFormPoint(x,y,z,source,dest)
EndFunction

Rem
bbdoc: Transforms the passed vector coordinates between coordinate systems
returns: Returns the transformed point as a vector3df
about: <i>Parameters</i>
<ul>
<li>x:Float - X coordinate of a vector in 3d space </li>
<li>y:Float - Y coordinate of a vector in 3d space </li>
<li>z:Float - Z coordinate of a vector in 3d space </li>
<li>source:ib3d_Entity - instance of source entity, or Null for 3d world</li>
<li>dest:ib3d_Entity - instance of destination entity, or Null for 3d world</li>
</ul>
After using TFormPoint the new coordinates can be read with ib3d_TFormedX(), ib3d_TFormedY() and ib3d_TFormedZ(). <br><br>
See ib3d_EntityX() for details about local coordinates. <br><br>
Similar to ib3d_TFormPoint, but operates on a vector. A vector can be thought of as 'displacement relative to current location'. <br><br>
For example, vector (1,2,3) means one step to the right, two steps up and three steps forward. <br><br>
This is analogous to ib3d_PositionEntity and ib3d_MoveEntity: <br><br>
ib3d_PositionEntity entity, x,y,z ; put entity at point (x,y,z) <br><br>
ib3d_MoveEntity entity, x,y,z ; add vector (x,y,z) to current position 
EndRem
Function ib3d_TFormVector:Vector3df(x:Float,y:Float,z:Float,source:ib3d_Entity,dest:ib3d_Entity)
	Local v:Vector3df=_VECTOR3DF(x,y,z)
	Rem
	Local m:Matrix4=Matrix4.create()
	m.setTranslation(Vector3df.createFromVals(x,y,z))
	Local pos:Vector3df=source._node.getPosition()
	m.translateVect(pos)
	'v=pos.Minus(source._node.getPosition())
	V=POS
	EndRem
	If source Then 
		source._node.getRelativeTransformation().transformVect(v)
		v=v.Minus(source._node.getPosition())
	EndIf
	
	If dest Then dest._node.getAbsoluteTransformation().transformVect(v)

	'DebugLog("after: "+v.getX()+" | "+v.getY()+" | "+v.getZ())
	Return v
EndFunction

Rem
bbdoc: Returns the X component of the last ib3d_TFormPoint, ib3d_TFormVector or ib3d_TFormNormal operation
returns: X component
EndRem
Function ib3d_TFormedX:Float()
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.TFormedX()
EndFunction

Rem
bbdoc: Returns the Y component of the last ib3d_TFormPoint, ib3d_TFormVector or ib3d_TFormNormal operation
returns: Y component
EndRem
Function ib3d_TFormedY:Float()
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.TFormedY()
EndFunction

Rem
bbdoc: Returns the Z component of the last ib3d_TFormPoint, ib3d_TFormVector or ib3d_TFormNormal operation
returns: Z component
EndRem
Function ib3d_TFormedZ:Float()
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	Return _g_ib3d_engine.TFormedZ()
EndFunction

Rem
bbdoc: Returns the X,Y,Z components of the last ib3d_TFormPoint, ib3d_TFormVector or ib3d_TFormNormal operation
about: <i>Parameters</i>
<ul>
<li>x:Float - variable to store the X coordinate of last TForm command</li>
<li>y:Float - variable to store the Y coordinate of last TForm command</li>
<li>z:Float - variable to store the Z coordinate of last TForm command</li>
</ul>
EndRem
Function ib3d_LastTFormed(x:Float Var,y:Float Var,z:Float Var)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	_g_ib3d_engine.LastTFormed(x,y,z)
EndFunction

Rem
bbdoc: Adds a GUI event handler to the engine
about: <i>Parameters</i>
<ul>
<li>element_id:Int - The id of the GUI element to handle events for</li>
<li>handler:ib3d_GUIEventHandler - instance of ib3d_GUIEventHandler that implements the code to handle the events</li>
</ul>
EndRem
Rem
Function ib3d_AddGUIEventHandler(element_id:Int, handler:ib3d_GUIEventHandler)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	_g_ib3d_engine.AddGUIEventHandler(element_id, handler)
EndFunction
EndRem
Rem
bbdoc: Removes a GUI event handler from the engine
about: <i>Parameters</i>
<ul>
<li>element_id:Int - The id of the GUI element to handle events for</li>
</ul>
EndRem
Rem
Function ib3d_RemoveGUIEventHandler(element_id:Int)
	If Not _g_ib3d_engine Then RuntimeError "iB3D engine has not been initialized"
	_g_ib3d_engine.RemoveGUIEventHandler(element_id)
EndFunction
EndRem

Rem
bbdoc: Constant for Left mouse button
EndRem
Const IB3D_MOUSE_LEFT:Int = 0
Rem
bbdoc: Constant for Right mouse button
EndRem
Const IB3D_MOUSE_RIGHT:Int = 1
Rem
bbdoc: Constant for Middle mouse button
EndRem
Const IB3D_MOUSE_MIDDLE:Int = 2

Rem
bbdoc: Event receiver for handling multiple inputs at one time
EndRem
Type ib3d_EventReceiver Extends IEventReceiver

	Field m_key_cnt:Int[256]
	Field m_key_buf:Int[256]
	Field m_key_buf_old:Int[256]
	Field m_key_up_down:Int[256]
	Field m_key_down_up:Int[256]
	Field m_key_shift:Int[256]
	Field m_key_ctrl:Int[256]
	Field m_key_menu:Int[256]
	
	Field m_mouse_button_cnt:Int[3]
	Field m_mouse_button_buf:Int[3]
	Field m_mouse_button_buf_old:Int[3]
	Field m_mouse_button_up_down:Int[3]
	Field m_mouse_button_down_up:Int[3]
	Field m_mouse_button_shift:Int[3]
	Field m_mouse_button_ctrl:Int[3]
	Field m_mouse_button_menu:Int[3]

	Field m_mouse_x:Int
	Field m_mouse_y:Int
	Field m_mouse_z:Float

	Rem
	bbdoc: Creates the smooth event receiver
	returns: An instance of smooth event receiver
	about:
	Pass the returned instance to the device to start receiving input.
	EndRem
	Function createReceiver:ib3d_EventReceiver()
		Return ib3d_EventReceiver(ib3d_EventReceiver.Create(ib3d_EventReceiver.generate))
	EndFunction

	Method OnEvent:Int(event:SEvent)
		
		Local event_type:Int=event.getEventType()
		
		If event_type=EET_KEY_INPUT_EVENT
			Local key:Int=event.getKeyInputKey()
		
			m_key_buf_old[key] = m_key_buf[key]
			m_key_buf[key] = event.getKeyPressedDown()

			' store the modifier flags
			m_key_shift[key] = event.getKeyShift()
			m_key_ctrl[key] = event.getKeyControl()
			m_key_menu[key] = IsKeyDown(EKEY_MENU) Or IsKeyDown(EKEY_LMENU) Or IsKeyDown(EKEY_RMENU)
			
			' store state changes
			m_key_up_down[key]=(m_key_buf[key] And Not m_key_buf_old[key])
			m_key_down_up[key]=(Not m_key_buf[key] And m_key_buf_old[key])

			' log a key hit if the key is down and it wasnt down before
			If m_key_up_down[key] Then m_key_cnt[key]:+1
			
		ElseIf event_type=EET_MOUSE_INPUT_EVENT
			Local eventType:Int = event.getMouseEventType()

			If eventType < EMIE_MOUSE_MOVED
				Local button:Int=(event.getMouseEventType() Mod 3)
				
				' store the old state
				m_mouse_button_buf_old[button] = m_mouse_button_buf[button]
				
				' set the current state
				m_mouse_button_buf[button] = Not (event.getMouseEventType()/3)

				' store state changes
				m_mouse_button_up_down[button]=(m_mouse_button_buf[button] And Not m_mouse_button_buf_old[button])
				m_mouse_button_down_up[button]=(Not m_mouse_button_buf[button] And m_mouse_button_buf_old[button])
				
				' log a hit if the state went from up to down
				If m_mouse_button_up_down[button] Then m_mouse_button_cnt[button]:+1
				
				' store the modifier flags
				m_mouse_button_shift[button] = IsKeyDown(EKEY_SHIFT) Or IsKeyDown(EKEY_LSHIFT) Or IsKeyDown(EKEY_RSHIFT)
				m_mouse_button_ctrl[button] = IsKeyDown(EKEY_CONTROL) Or IsKeyDown(EKEY_LCONTROL) Or IsKeyDown(EKEY_RCONTROL)
				m_mouse_button_menu[button] = IsKeyDown(EKEY_MENU) Or IsKeyDown(EKEY_LMENU) Or IsKeyDown(EKEY_RMENU)
			ElseIf eventType = EMIE_MOUSE_MOVED
				m_mouse_x = event.getMouseX()
				m_mouse_y = event.getMouseY()
			ElseIf eventType = EMIE_MOUSE_WHEEL
				m_mouse_z = event.getMouseWheel()
			EndIf
		ElseIf event_type = EET_GUI_EVENT
			If OnGUIEvent(event.getGUIEventCaller(),event.getGUIEventType()) Then Return True				
		EndIf
		
		Return _g_ib3d_engine.OnEvent(event)
	EndMethod

	Function generate:IEventReceiver()
		Return New ib3d_EventReceiver
	EndFunction

	' ------------------------------------------------
	' keyboard functions
	' ------------------------------------------------
		
	Rem
	bbdoc: Returns the # of times a key has been hit since the last clear
	returns: Integer count
	about: <i>Parameters</i>
	<ul>
	<li>index:Int - The key to check.</li>
	<li>bClear:Int - Pass True to clear out the hit count.  Defaults to False</li>
	</ul>
	EndRem
	Method KeyHitCnt:Int(index:Int,bClear:Int=False)
		Local retval:Int=m_key_cnt[index]
		If bClear Then m_key_cnt[index]=0
		Return retval
	EndMethod
	
	Rem
	bbdoc: Checks to see if a key is currently down
	returns: True if the key is currently down
	about: <i>Parameters</i>
	<ul>
	<li>index:Int - The key to check.</li>
	<li>bCheckShift:Int - True to only return true if a Shift key is pressed as well.  Defaults to False.</li>
	<li>bCheckCtrl:Int - True to only return true if a CTRL key is pressed as well.  Defaults to False.</li>
	<li>bCheckMenu:Int - True to only return true if a Menu (ALT) key is pressed as well.  Defaults to False.</li>
	</ul>
	EndRem
	Method IsKeyDown:Int(index:Int,bCheckShift:Int=False,bCheckCtrl:Int=False,bCheckMenu:Int=False) 
		' check the key itself
		Local bKeyDown:Int=m_key_buf[index]
		' check the shift modifier
		bKeyDown = bKeyDown And (Not bCheckShift Or m_key_shift[index])
		' check the control modifier
		bKeyDown = bKeyDown And (Not bCheckCtrl Or m_key_ctrl[index])
		' check the menu  modifier
		bKeyDown = bKeyDown And (Not bCheckMenu Or m_key_menu[index])
		
		Return bKeyDown
	EndMethod
	
	Rem
	bbdoc: Checks to see if a key state changed from up to down
	returns: True if the state changed
	about: <i>Parameters</i>
	<ul>
	<li>index:Int - The key to check.</li>
	<li>bClear:Int - False to not clear the up down state.  Defaults to True.</li>
	</ul>
	EndRem
	Method IsKeyUpDown:Int(index:Int,bClear:Int=True) 
		Local bIsKeyUpDown:Int=m_key_up_down[index]
		If bClear Then m_key_up_down[index]=False
		Return bIsKeyUpDown 
	EndMethod 
	
	Rem
	bbdoc: Checks to see if a key state changed from down to up
	returns: True if the state changed
	about: <i>Parameters</i>
	<ul>
	<li>index:Int - The key to check.</li>
	<li>bClear:Int - False to not clear the down up state.  Defaults to True.</li>
	</ul>
	EndRem
	Method IsKeyDownUp:Int(index:Int,bClear:Int=True) 
		Local bIsKeyDownUp:Int=m_key_down_up[index]
		If bClear Then m_key_down_up[index]=False
		Return bIsKeyDownUp
	EndMethod 

	Rem
	bbdoc: Changes all key states to False
	EndRem
	Method FlushKeys()
		m_key_cnt=New Int[256]
		m_key_buf=New Int[256]
		m_key_buf_old=New Int[256]
		m_key_up_down=New Int[256]
		m_key_down_up=New Int[256]
		m_key_shift=New Int[256]
		m_key_ctrl=New Int[256]
		m_key_menu=New Int[256]
	EndMethod

	' ------------------------------------------------
	' mouse functions
	' ------------------------------------------------

	Rem
	bbdoc: Returns the # of times a mouse button has been hit since the last clear
	returns: Integer count
	about: <i>Parameters</i>
	<ul>
	<li>index:Int - The mouse button to check to check.  (see SER_MOUSE constants)</li>
	<li>bClear:Int - Pass True to clear out the hit count.  Defaults to False</li>
	</ul>
	EndRem
	Method MouseHitCnt:Int(index:Int,bClear:Int=False)
		Local retval:Int=m_mouse_button_cnt[index]
		If bClear Then m_mouse_button_cnt[index]=0
		Return retval
	EndMethod
		
	Rem
	bbdoc: Checks to see if a mouse button is currently down
	returns: True if the mouse button is currently down
	about: <i>Parameters</i>
	<ul>
	<li>index:Int - The mouse button to check to check.  (see SER_MOUSE constants)</li>
	<li>bCheckShift:Int - True to only return true if a Shift key is pressed as well.  Defaults to False.</li>
	<li>bCheckCtrl:Int - True to only return true if a CTRL key is pressed as well.  Defaults to False.</li>
	<li>bCheckMenu:Int - True to only return true if a Menu (ALT) key is pressed as well.  Defaults to False.</li>
	</ul>
	EndRem
	Method IsMouseButtonDown:Int(index:Int,bCheckShift:Int=False,bCheckCtrl:Int=False,bCheckMenu:Int=False) 
		' check the key itself
		Local bButtonDown:Int=m_mouse_button_buf[index]
		' check the shift modifier
		bButtonDown = bButtonDown And (Not bCheckShift Or m_mouse_button_shift[index])
		' check the control modifier
		bButtonDown =  bButtonDown And (Not bCheckCtrl Or m_mouse_button_ctrl[index])
		' check the menu  modifier
		bButtonDown = bButtonDown And (Not bCheckMenu Or m_mouse_button_menu[index])
		
		Return bButtonDown		
	EndMethod 
	
	Rem
	bbdoc: Checks to see if a mouse button state changed from up to down
	returns: True if the state changed
	about: <i>Parameters</i>
	<ul>
	<li>index:Int - The mouse button to check to check.  (see SER_MOUSE constants)</li>
	<li>bClear:Int - False to not clear the up down state.  Defaults to True.</li>
	</ul>
	EndRem
	Method IsMouseButtonUpDown:Int(index:Int,bClear:Int=True) 
		Local bIsMouseUpDown:Int=m_mouse_button_up_down[index]
		If bClear Then m_mouse_button_up_down[index]=False
		Return bIsMouseUpDown
	EndMethod 
	
	Rem
	bbdoc: Checks to see if a mouse button state changed from down to up
	returns: True if the state changed
	about: <i>Parameters</i>
	<ul>
	<li>index:Int - The mouse button to check to check.  (see SER_MOUSE constants)</li>
	<li>bClear:Int - False to not clear the down up state.  Defaults to True.</li>
	</ul>
	EndRem
	Method IsMouseButtonDownUp:Int(index:Int,bClear:Int=True) 
		Local bIsMouseDownUp:Int=m_mouse_button_down_up[index]
		If bClear Then m_mouse_button_down_up[index]=False
		Return bIsMouseDownUp
	EndMethod 
	
	Rem
	bbdoc: Returns the X position of the mouse
	returns: X position as an integer
	EndRem
	Method MouseX:Int()
		Return m_mouse_x
	EndMethod

	Rem
	bbdoc: Returns the Y position of the mouse
	returns: Y position as an integer
	EndRem
	Method MouseY:Int()
		Return m_mouse_y
	EndMethod

	Rem
	bbdoc: Returns the Z (scrollwheel) position of the mouse
	returns: The scrollwheel position represented as a Float between -1.0 and 1.0
	about: NOTE: I have found this to be pretty flaky so use with care.
	EndRem
	Method MouseZ:Float()
		Return m_mouse_z
	EndMethod
	
	Rem
	bbdoc: Changes all mouse_button states to False
	EndRem
	Method FlushMouse()
		m_mouse_button_cnt=New Int[3]
		m_mouse_button_buf=New Int[3]
		m_mouse_button_buf_old=New Int[3]
		m_mouse_button_up_down=New Int[3]
		m_mouse_button_down_up=New Int[3]
		m_mouse_button_shift=New Int[3]
		m_mouse_button_ctrl=New Int[3]
		m_mouse_button_menu=New Int[3]
	EndMethod

	Method OnGUIEvent:Int(element:IGUIElement, event_type:Int) 
		Return False
	EndMethod
	
EndType

Rem
Type ib3d_EventReceiver Extends IEventReceiver

	Field m_key_cnt:Int[256]
	Field m_key_buf:Int[256]
	Field m_key_buf_old:Int[256]
	Field m_mouse_button_buf:Int[3]
	Field m_mouse_button_buf_old:Int[3]

	Field m_event_handlers:TMap = Null

	Method OnEvent:Int(event:SEvent)
		
		Local event_type:Int = event.getEventType()
		
		If event_type = EET_KEY_INPUT_EVENT
			If event.getKeyPressedDown() Then m_key_cnt[event.getKeyInputKey()]:+1				
			m_key_buf_old[event.getKeyInputKey()] = m_key_buf[event.getKeyInputKey()]
			m_key_buf[event.getKeyInputKey()] = event.getKeyPressedDown() 
		ElseIf event_type = EET_MOUSE_INPUT_EVENT
			If event.getMouseEventType() < EMIE_MOUSE_MOVED
				m_mouse_button_buf_old[event.getMouseEventType() Mod 3] = m_mouse_button_buf[event.getMouseEventType() Mod 3]
				If (event.getMouseEventType()/3) Then m_mouse_button_buf[event.getMouseEventType() Mod 3] = False Else m_mouse_button_buf[event.getMouseEventType() Mod 3]=True
			EndIf
		ElseIf event_type = EET_GUI_EVENT
			If OnGUIEvent(event.getGUIEventCaller(),event.getGUIEventType()) Then Return True

'			If m_event_handlers Then
'				Local element:IGUIElement = event.getGUIEventCaller()
'				' find a handler for the gui element
'				Local handler:ib3d_GUIEventHandler = ib3d_GUIEventHandler(m_event_handlers.ValueForKey(String.FromInt(element.getID())))
'				' if we have one, then call it
'				If handler Then 
'					handler.OnEvent(ib3d_GUIElement.findElement(element), event.getGUIEventType())
'				EndIf
'			EndIf						
		EndIf
		
		Return _g_ib3d_engine.OnEvent(event)
	EndMethod

	Function generate:IEventReceiver()
		Return IEventReceiver(New ib3d_EventReceiver)
	EndFunction

	Method KeyHitCnt:Int(index:Int,bClear:Int=False)
		Local retval:Int=m_key_cnt[index]
		If bClear Then m_key_cnt[index]=0
		Return retval
	EndMethod
	
	Method IsKeyDown:Int(index:Int) 
		Return m_key_buf[index]
	EndMethod
	
	Method IsKeyUpDown:Int(index:Int) 
		Return (m_key_buf[index] And Not m_key_buf_old[index])
	EndMethod 
	
	Method IsKeyDownUp:Int(index:Int) 
		Return (Not m_key_buf[index] And m_key_buf_old[index])
	EndMethod  
	
	Method IsMouseButtonDown:Int(index:Int) 
		Return m_mouse_button_buf[index]
	EndMethod 
	
	Method IsMouseButtonUpDown:Int(index:Int) 
		Return (m_mouse_button_buf[index] And Not m_mouse_button_buf_old[index])
	EndMethod 
	
	Method IsMouseButtonDownUp:Int(index:Int) 
		Return (Not m_mouse_button_buf[index] And m_mouse_button_buf_old[index])
	EndMethod 

	Method OnGUIEvent:Int(element:IGUIElement, event_type:Int) 
		Return False
	EndMethod

'	Method AddGUIEventHandler(element_id:Int, handler:ib3d_GUIEventHandler)
'		If Not m_event_handlers Then m_event_handlers = New TMap
'		m_event_handlers.Insert(String.FromInt(element_id),handler)
'	EndMethod

'	Method RemoveGUIEventHandler(element_id:Int)
'		m_event_handlers.Remove(String.FromInt(element_id))
'	EndMethod

EndType

EndRem
