' ------------------------------------------------------------------------------------
' irr::core::line2d<f32> wrappers
' ------------------------------------------------------------------------------------

Public

Type Line2df Extends IrrBase Final

	'--------------------------------------------
	' operator overloads
	'--------------------------------------------
	
	Method Plus:Line2df(pPoint:Vector2df)
		Return createFromHandle(IrrCore_Line2df_Plus(handle,pPoint.handle))
	EndMethod
	Method Minus:Line2df(pPoint:Vector2df)
		Return createFromHandle(IrrCore_Line2df_Minus(handle,pPoint.handle))
	EndMethod
	Method PlusEq(pPoint:Vector2df)
		IrrCore_Line2df_PlusEq(handle,pPoint.handle)
	EndMethod
	Method MinusEq(pPoint:Vector2df)
		IrrCore_Line2df_MinusEq(handle,pPoint.handle)
	EndMethod

	Method EqEq:Int(pLine2:Line2df)
		Return IrrCore_Line2df_EqEq:Int(handle,pLine2.handle)
	EndMethod

	Method NotEq:Int(pLine2:Line2df)
		Return IrrCore_Line2df_EqEq:Int(handle,pLine2.handle)
	EndMethod

	'--------------------------------------------
	' set/get routines
	'--------------------------------------------	

	Method setLineFromVals(xa:Int,ya:Int,xb:Int,yb:Int)
		IrrCore_Line2df_setLineFromVals(handle,xa,ya,xb,yb)
	EndMethod
	
	Method setLineFromVects(pStart:Vector2df,pEnd:Vector2df)
		IrrCore_Line2df_setLineFromVects(handle,pStart.handle,pEnd.handle)
	EndMethod
	
	Method setLineFromLine(pOther:Line2df)
		IrrCore_Line2df_setLineFromLine(handle,pOther.handle)
	EndMethod

	Method getLength:Double()
		Return IrrCore_Line2df_getLength(handle)
	EndMethod
	
	Method getVector:Vector2df()
		Return Vector2df.createFromHandle(IrrCore_Line2df_getVector(handle))
	EndMethod

	Method getUnitVector:Vector2df()
		Return Vector2df.createFromHandle(IrrCore_Line2df_getUnitVector(handle))
	EndMethod

	Method getAngleWith:Double(pLine2:Line2df)
		Return IrrCore_Line2df_getAngleWith(handle,pLine2.handle)
	EndMethod
	
	Method intersectWith:Int(pLine2:Line2df,pOut:Vector2df)
		Return IrrCore_Line2df_intersectWith(handle,pLine2.handle,pOut.handle)
	EndMethod

	Method getStart:Vector2df()
		' set to not delete
		Return Vector2df.createFromHandle(IrrCore_Line2df_getStart(handle),False)
	EndMethod
	Method getEnd:Vector2df()
		' set to not delete
		Return Vector2df.createFromHandle(IrrCore_Line2df_getEnd(handle),False)
	EndMethod

	Method getLengthSQ:Float()
		Return IrrCore_Line2df_getLengthSQ(handle)
	EndMethod
	
	Method getMiddle:Vector2df()
		Return Vector2df.createFromHandle(IrrCore_Line2df_getMiddle(handle),True)
	EndMethod
	
	Method getPointOrientation:Float(point:Vector2df)
		Return IrrCore_Line2df_getPointOrientation(handle,point.handle)
	EndMethod
	
	Method isPointOnLine:Int(point:Vector2df)
		Return IrrCore_Line2df_isPointOnLine(handle,point.handle)
	EndMethod
	
	Method isPointBetweenStartAndEnd:Int(point:Vector2df)
		Return IrrCore_Line2df_isPointBetweenStartAndEnd(handle,point.handle)
	EndMethod
	
	' must delete
	Method getClosestPoint:Vector2df(point:Vector2df)
		Return Vector2df.createFromHandle(IrrCore_Line2df_getClosestPoint(handle,point.handle),True)
	EndMethod

	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------
	
	Function create:Line2df()
		Return createFromHandle(IrrCore_Line2df_new())
	EndFunction
	
	Function createFromVals:Line2df(xa:Int=0,ya:Int=0,xb:Int=0,yb:Int=0)
		Return createFromHandle(IrrCore_Line2df_newFromVals(xa,ya,xb,yb))
	EndFunction
	
	Function createFromVects:Line2df(pStart:Vector2df,pEnd:Vector2df)	
		Return createFromHandle(IrrCore_Line2df_newFromVects(pStart.handle,pEnd.handle))
	EndFunction
	
	Function createFromLine:Line2df(pOther:Line2df)
		Return createFromHandle(IrrCore_Line2df_newFromLine(pOther.handle))
	EndFunction

	Function createFromHandle:Line2df(pLine:Int,bMustDel:Int=True)
		If Not pLine Then Return Null
		Local oRetVal:Line2df=New Line2df
		oRetVal.attach(pLine,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrCore_Line2df_del(handle)
	EndMethod		
EndType

