' ------------------------------------------------------------------------------------
' irr::core::line2d<s32> wrappers
' ------------------------------------------------------------------------------------
Public

Type Line2di Extends IrrBase Final

	'--------------------------------------------
	' operator overloads
	'--------------------------------------------
	
	Method Plus:Line2di(pPoint:Vector2di)
		Return createFromHandle(IrrCore_Line2di_Plus(handle,pPoint.handle))
	EndMethod
	Method Minus:Line2di(pPoint:Vector2di)
		Return createFromHandle(IrrCore_Line2di_Minus(handle,pPoint.handle))
	EndMethod
	Method PlusEq(pPoint:Vector2di)
		IrrCore_Line2di_PlusEq(handle,pPoint.handle)
	EndMethod
	Method MinusEq(pPoint:Vector2di)
		IrrCore_Line2di_MinusEq(handle,pPoint.handle)
	EndMethod

	Method EqEq:Int(pLine2:Line2di)
		Return IrrCore_Line2di_EqEq:Int(handle,pLine2.handle)
	EndMethod

	Method NotEq:Int(pLine2:Line2di)
		Return IrrCore_Line2di_EqEq:Int(handle,pLine2.handle)
	EndMethod

	'--------------------------------------------
	' set/get routines
	'--------------------------------------------	

	Method setLineFromVals(xa:Int,ya:Int,xb:Int,yb:Int)
		IrrCore_Line2di_setLineFromVals(handle,xa,ya,xb,yb)
	EndMethod
	
	Method setLineFromVects(pStart:Vector2di,pEnd:Vector2di)
		IrrCore_Line2di_setLineFromVects(handle,pStart.handle,pEnd.handle)
	EndMethod
	
	Method setLineFromLine(pOther:Line2di)
		IrrCore_Line2di_setLineFromLine(handle,pOther.handle)
	EndMethod

	Method getLength:Double()
		Return IrrCore_Line2di_getLength(handle)
	EndMethod
	
	Method getVector:Vector2di()
		Return Vector2di.createFromHandle(IrrCore_Line2di_getVector(handle))
	EndMethod

	Method getUnitVector:Vector2di()
		Return Vector2di.createFromHandle(IrrCore_Line2di_getUnitVector(handle))
	EndMethod

	Method getAngleWith:Double(pLine2:Line2di)
		Return IrrCore_Line2di_getAngleWith(handle,pLine2.handle)
	EndMethod
	
	Method intersectWith:Int(pLine2:Line2di,pOut:Vector2di)
		Return IrrCore_Line2di_intersectWith(handle,pLine2.handle,pOut.handle)
	EndMethod

	Method getStart:Vector2di()
		' set to not delete
		Return Vector2di.createFromHandle(IrrCore_Line2di_getStart(handle),False)
	EndMethod
	Method getEnd:Vector2di()
		' set to not delete
		Return Vector2di.createFromHandle(IrrCore_Line2di_getEnd(handle),False)
	EndMethod

	Method getLengthSQ:Int()
		Return IrrCore_Line2di_getLengthSQ(handle)
	EndMethod
	
	Method getMiddle:Vector2di()
		' must delete
		Return Vector2di.createFromHandle(IrrCore_Line2di_getMiddle(handle),True)
	EndMethod
	
	Method getPointOrientation:Int(point:Vector2di)
		Return IrrCore_Line2di_getPointOrientation(handle,point.handle)
	EndMethod
	
	Method isPointOnLine:Int(point:Vector2di)
		Return IrrCore_Line2di_isPointOnLine(handle,point.handle)
	EndMethod
	
	Method isPointBetweenStartAndEnd:Int(point:Vector2di)
		Return IrrCore_Line2di_isPointBetweenStartAndEnd(handle,point.handle)
	EndMethod
	
	' must delete
	Method getClosestPoint:Vector2di(point:Vector2di)
		Return Vector2di.createFromHandle(IrrCore_Line2di_getClosestPoint(handle,point.handle),True)
	EndMethod
	
	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------
	
	Function create:Line2di()
		Return createFromHandle(IrrCore_Line2di_new())
	EndFunction
	
	Function createFromVals:Line2di(xa:Int=0,ya:Int=0,xb:Int=0,yb:Int=0)
		Return createFromHandle(IrrCore_Line2di_newFromVals(xa,ya,xb,yb))
	EndFunction
	
	Function createFromVects:Line2di(pStart:Vector2di,pEnd:Vector2di)	
		Return createFromHandle(IrrCore_Line2di_newFromVects(pStart.handle,pEnd.handle))
	EndFunction
	
	Function createFromLine:Line2di(pOther:Line2di)
		Return createFromHandle(IrrCore_Line2di_newFromLine(pOther.handle))
	EndFunction

	Function createFromHandle:Line2di(pLine:Int,bMustDel:Int=True)
		If Not pLine Then Return Null
		Local oRetVal:Line2di=New Line2di
		oRetVal.attach(pLine,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrCore_Line2di_del(handle)
	EndMethod		
EndType