' ------------------------------------------------------------------------------------
'  irr::scene::ISceneManager wrappers
' ------------------------------------------------------------------------------------
Public

Type ISceneManager Extends IReferenceCounted Final

	Function createFromHandle:ISceneManager(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:ISceneManager=New ISceneManager
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Method createTextureAnimator:ISceneNodeAnimator(textures:ITexture[],timePerFrame:Int,bLoop:Int=True)
		Local texarr_size:Int=0
		Local texarr:Int[]=GetHandleArray(textures)
		If texarr Then texarr_size=texarr.Dimensions()[0]
	
		Return ISceneNodeAnimator.createFromHandle(IrrScene_ISceneManager_createTextureAnimator(handle,texarr,texarr_size,timePerFrame,bLoop),True)
	EndMethod

	Method createFollowSplineAnimator:ISceneNodeAnimator(startTime:Int,points:Vector3df[],speed:Float=1.0,tightness:Float=0.5)
		Local point_cnt:Int=points.Dimensions()[0]
		Local point_array:Int[]=GetHandleArray(points)
		
		Return ISceneNodeAnimator.createFromHandle(IrrScene_ISceneManager_createFollowSplineAnimator(handle,startTime,point_array,point_cnt,speed,tightness),True)
	EndMethod
	
	Method clear()
		IrrScene_ISceneManager_clear(handle)
	EndMethod
	
	Method postEventFromUser:Int(usrevent:SEvent)
		Return IrrScene_ISceneManager_postEventFromUser(handle,usrevent.handle)
	EndMethod
	
	Method addtoDeletionQueue(node:ISceneNode)
		IrrScene_ISceneManager_addToDeletionQueue(handle,node.handle)
	EndMethod
	
	Method getMeshManipulator:IMeshManipulator()
		Return IMeshManipulator.createFromHandle(IrrScene_ISceneManager_getMeshManipulator(handle),False)
	EndMethod
	
	Method getSceneCollisionManager:ISceneCollisionManager()
		Return ISceneCollisionManager.createFromHandle(IrrScene_ISceneManager_getSceneCollisionManager(handle),False)
	EndMethod

	Method addExternalMeshLoader(loader:IMeshLoader)
		IrrScene_ISceneManager_addExternalMeshLoader(handle,loader.handle)
	EndMethod
	
	Method createMetaTriangleSelector:IMetaTriangleSelector()	
		Return IMetaTriangleSelector.createFromHandle(IrrScene_ISceneManager_createMetaTriangleSelector(handle),True)
	EndMethod

	Method createTerrainTriangleSelector:ITriangleSelector(node:ITerrainSceneNode,LOD:Int=0)
		Return ITriangleSelector.createFromHandle(IrrScene_ISceneManager_createTerrainTriangleSelector(handle,node.handle,LOD),True)
	EndMethod

	Method createOctTreeTriangleSelector:ITriangleSelector(mesh:IMesh,node:ISceneNode,minimalPolysPerNode:Int=32)
		Return ITriangleSelector.createFromHandle(IrrScene_ISceneManager_createOctTreeTriangleSelector:Int(handle,mesh.handle,node.handle,minimalPolysPerNode),True)
	EndMethod
	
	Method createTriangleSelectorFromBoundingBox:ITriangleSelector(node:ISceneNode)
		Return ITriangleSelector.createFromHandle(IrrScene_ISceneManager_createTriangleSelectorFromBoundingBox(handle,node.handle),True)
	EndMethod

	Method createTriangleSelector:ITriangleSelector(mesh:IMesh,node:ISceneNode) 
		Return ITriangleSelector.createFromHandle(IrrScene_ISceneManager_createTriangleSelector(handle,mesh.handle,node.handle),True)
	EndMethod

	Method createTriangleSelectorFromAnimatedNode:ITriangleSelector(node:IAnimatedMeshSceneNode)
		Return ITriangleSelector.createFromHandle(IrrScene_ISceneManager_createTriangleSelectorFromAnimatedNode(handle, _HANDLE(node)), True)
	EndMethod

	Method createCollisionResponseAnimator:ISceneNodeAnimatorCollisionResponse(world:ITriangleSelector,sceneNode:ISceneNode,ellipsoidRadius:Vector3df=Null,gravityPerSecond:Vector3df=Null,ellipsoidTranslation:Vector3df=Null,slidingValue:Float=0.0005)
		If ellipsoidRadius=Null Then ellipsoidRadius=New Vector3df ' default handle to 0
		If gravityPerSecond=Null Then gravityPerSecond=New Vector3df ' default handle to 0
		If ellipsoidTranslation=Null Then ellipsoidTranslation=New Vector3df ' default handle to 0
		Return ISceneNodeAnimatorCollisionResponse.createFromHandle(IrrScene_ISceneManager_createCollisionResponseAnimator:Int(handle,world.handle,sceneNode.handle,ellipsoidRadius.handle,gravityPerSecond.handle,ellipsoidTranslation.handle,slidingValue),True)
	EndMethod

	Method createDeleteAnimator:ISceneNodeAnimator(timeMs:Int)
		Return ISceneNodeAnimator.createFromHandle(IrrScene_ISceneManager_createDeleteAnimator(handle,timeMs),True)
	EndMethod

	Method createFlyStraightAnimator:ISceneNodeAnimator(startPoint:Vector3df,endPoint:Vector3df,timeForWay:Int,bLoop:Int=False)
		Return ISceneNodeAnimator.createFromHandle(IrrScene_ISceneManager_createFlyStraightAnimator(handle,startPoint.handle,endPoint.handle,timeForWay,bLoop),True)
	EndMethod

	Method createFlyCircleAnimator:ISceneNodeAnimator(center:Vector3df = Null,radius:Float = 100, speed:Float=0.001, direction:Vector3df = Null, startPosition:Float = 0.0, radiusEllipsoid:Float = 0.0)
		Return ISceneNodeAnimator.createFromHandle(IrrScene_ISceneManager_createFlyCircleAnimator(handle,_HANDLE(center),radius,speed,_HANDLE(direction), startPosition, radiusEllipsoid),True)
	EndMethod

	Method createRotationAnimator:ISceneNodeAnimator(rotationSpeed:Vector3df)
		Return ISceneNodeAnimator.createFromHandle(IrrScene_ISceneManager_createRotationAnimator(handle,_HANDLE(rotationSpeed)),True)
	EndMethod

	Method registerNodeForRendering(node:ISceneNode,E_SCENE_NODE_RENDER_PASS_pass:Int=ESNRP_AUTOMATIC)
		IrrScene_ISceneManager_registerNodeForRendering(handle,node.handle,E_SCENE_NODE_RENDER_PASS_pass)
	EndMethod

	Method getShadowColor:SColor()
		Return SColor.createFromHandle(IrrScene_ISceneManager_getShadowColor(handle))
	EndMethod

	Method setShadowColor(color:SColor=Null)
		If color=Null Then color=New SColor ' default handle to 0
		IrrScene_ISceneManager_setShadowColor(handle,color.handle)	
	EndMethod

	Method setActiveCamera(camera:ICameraSceneNode)
		IrrScene_ISceneManager_setActiveCamera(handle,camera.handle)
	EndMethod
	
	Method getActiveCamera:ICameraSceneNode()
		Return ICameraSceneNode.createFromHandle(IrrScene_ISceneManager_getActiveCamera(handle))
	EndMethod
	
	Method getSceneNodeFromId:ISceneNode(id:Int,start:ISceneNode)
		Return ISceneNode.createFromHandle(IrrScene_ISceneManager_getSceneNodeFromId(handle,id,start.handle),False)
	EndMethod
	
	Method getSceneNodeFromName:ISceneNode(name:String,start:ISceneNode)
		Return ISceneNode.createFromHandle(IrrScene_ISceneManager_getSceneNodeFromName(handle,name,start.handle),False)
	EndMethod

	Method getRootSceneNode:ISceneNode()	
		Return ISceneNode.createFromHandle(IrrScene_ISceneManager_getRootSceneNode(handle),False)
	EndMethod
	
	Method addTerrainMesh:IAnimatedMesh(meshname:String,texture:ITexture,heightmap:IImage,stretch:Dimension2df=Null,maxHeight:Float=200.0,defBlock:Dimension2di=Null)
		If stretch=Null Then stretch=New Dimension2df ' default handle to 0
		If defBlock=Null Then defBlock=New Dimension2di ' default handle to 0
		Return IAnimatedMesh.createFromHandle(IrrScene_ISceneManager_addTerrainMesh(handle,meshname,texture.handle,heightmap.handle,stretch.handle,maxHeight,defBlock.handle),False)
	EndMethod

	Method addTerrainSceneNode:ITerrainSceneNode(heightMapFile:IReadFile,parent:ISceneNode=Null,id:Int=-1,pos:Vector3df=Null,rot:Vector3df=Null,scale:Vector3df=Null,VertexColor:SColor=Null,maxLOD:Int=5,patchSize:Int=ETPS_17,smoothFactor:Int=0,bAddAlsoIfHeightmapEmpty:Int = False)
		If parent=Null Then parent=New ISceneNodeDefault ' default handle to 0
		If pos=Null Then pos=New Vector3df ' default handle to 0
		If rot=Null Then rot=New Vector3df ' default handle to 0
		If scale=Null Then scale=New Vector3df ' default handle to 0
		If VertexColor=Null Then VertexColor=New SColor ' default handle to 0

		Return ITerrainSceneNode.createFromHandle(IrrScene_ISceneManager_addTerrainSceneNode(handle,heightMapFile.handle,parent.handle,id,pos.handle,rot.handle,scale.handle,VertexColor.handle,maxLOD,patchSize,smoothFactor,bAddAlsoIfHeightmapEmpty),False)
	EndMethod

	Method addTerrainSceneNodeFromFile:ITerrainSceneNode(heightMapFileName:String,parent:ISceneNode=Null,id:Int=-1,pos:Vector3df=Null,rot:Vector3df=Null,scale:Vector3df=Null,VertexColor:SColor=Null,maxLOD:Int=5,patchSize:Int=ETPS_17,smoothFactor:Int=0,bAddAlsoIfHeightmapEmpty:Int = False)
		If parent=Null Then parent=New ISceneNodeDefault ' default handle to 0
		If pos=Null Then pos=New Vector3df ' default handle to 0
		If rot=Null Then rot=New Vector3df ' default handle to 0
		If scale=Null Then scale=New Vector3df ' default handle to 0
		If VertexColor=Null Then VertexColor=New SColor ' default handle to 0

		Return ITerrainSceneNode.createFromHandle(IrrScene_ISceneManager_addTerrainSceneNodeFromFile(handle,heightMapFileName,parent.handle,id,pos.handle,rot.handle,scale.handle,VertexColor.handle,maxLOD,patchSize,smoothFactor,bAddAlsoIfHeightmapEmpty),False)
	EndMethod
	
	Method addHillPlaneMesh:IAnimatedMesh(name:String,tileSize:Dimension2df,tileCount:Dimension2di,material:SMaterial=Null,hillHeight:Float=0.0,countHills:Dimension2df=Null,textureRepeatCount:Dimension2df=Null)
		If material=Null Then material=New SMaterial ' default handle to 0
		If countHills=Null Then countHills=New Dimension2df ' default handle to 0
		If textureRepeatCount=Null Then textureRepeatCount=New Dimension2df ' default handle to 0
		Return IAnimatedMesh.createFromHandle(IrrScene_ISceneManager_addHillPlaneMesh(handle,name,tileSize.handle,tileCount.handle,material.handle,hillHeight,countHills.handle,textureRepeatCount.handle),False)
	EndMethod

	Method addTextSceneNode:ITextSceneNode(font:IGUIFont,text:String,color:SColor=Null,parent:ISceneNode=Null,pos:Vector3df=Null,id:Int=-1)
		If color=Null Then color=New SColor ' default handle to 0
		If parent=Null Then parent=New ISceneNodeDefault ' default handle to 0
		If pos=Null Then pos=New Vector3df ' default handle to 0
		
		Return ITextSceneNode.createFromHandle(IrrScene_ISceneManager_addTextSceneNode:Int(handle,font.handle,text,color.handle,parent.handle,pos.handle,id),False)
	EndMethod
	
	Method addDummyTransformationSceneNode:IDummyTransformationSceneNode(parent:ISceneNode,id:Int=-1)
		Return IDummyTransformationSceneNode.createFromHandle(IrrScene_ISceneManager_addDummyTransformationSceneNode(handle,parent.handle,id),False)
	EndMethod

	Method addEmptySceneNode:ISceneNode(parent:ISceneNode=Null,id:Int=-1)
		If parent=Null Then parent=New ISceneNodeDefault ' default handle to 0
		Return ISceneNode.createFromHandle(IrrScene_ISceneManager_addEmptySceneNode(handle,parent.handle,id),False)
	EndMethod
	
	Method addParticleSystemSceneNode:IParticleSystemSceneNode(bWithDefaultEmitter:Int=True,parent:ISceneNode=Null,id:Int=-1,pos:Vector3df=Null,rot:Vector3df=Null,scale:Vector3df=Null)
		If parent=Null Then parent=New ISceneNodeDefault ' default handle to 0
		If pos=Null Then pos=New Vector3df ' default handle to 0
		If rot=Null Then rot=New Vector3df ' default handle to 0
		If scale=Null Then scale=New Vector3df ' default handle to 0
		Return IParticleSystemSceneNode.createFromHandle(IrrScene_ISceneManager_addParticleSystemSceneNode(handle,bWithDefaultEmitter,parent.handle,id,pos.handle,rot.handle,scale.handle),False)
	EndMethod
	
	Method addSkyBoxSceneNode:ISceneNode(top:ITexture,bottom:ITexture,tLeft:ITexture,tRight:ITexture,front:ITexture,back:ITexture,parent:ISceneNode=Null,id:Int=-1)
		If parent=Null Then parent=New ISceneNodeDefault ' default handle to 0
		Return ISceneNode.createFromHandle(IrrScene_ISceneManager_addSkyBoxSceneNode(handle,top.handle,bottom.handle,tLeft.handle,tRight.handle,front.handle,back.handle,parent.handle,id),False)
	EndMethod

	Method addBillboardSceneNode:IBillboardSceneNode(parent:ISceneNode=Null,size:Dimension2df=Null,pos:Vector3df=Null,id:Int=-1,shade_top:SCOlor=Null,shade_down:SColor=Null)
		If parent=Null Then parent=New ISceneNodeDefault ' default handle to 0
		If size=Null size=New Dimension2df ' default handle to 0
		If pos=Null Then pos=New Vector3df ' default handle to 0
		If Not shade_top Then shade_top=New SColor
		If Not shade_down Then shade_down=New SColor
		Return IBillboardSceneNode.createFromHandle(IrrScene_ISceneManager_addBillboardSceneNode(handle,parent.handle,size.handle,pos.handle,id,shade_top.handle,shade_down.handle),False)
	EndMethod

	Method addLightSceneNode:ILightSceneNode(parent:ISceneNode=Null,pos:Vector3df=Null,color:SColorf=Null,radius:Float=100.0,id:Int=-1)
		If parent=Null Then parent=New ISceneNodeDefault ' default handle to 0
		If pos=Null Then pos=New Vector3df ' default handle to 0
		If color=Null Then color=New SColorf ' default handle to 0
		Return ILightSceneNode.createFromHandle(IrrScene_ISceneManager_addLightSceneNode(handle,parent.handle,pos.handle,color.handle,radius,id))
	EndMethod
	
	Method addCameraSceneNodeFPS:ICameraSceneNode(parent:ISceneNode=Null,rotateSpeed:Float=100.0,moveSpeed:Float=.5,id:Int=-1,keyMapArray:SKeyMap[]=Null,bNoVerticalMovement:Int=False,jumpSpeed:Float = 0.0, bInvertMouse:Int = False)
		If parent=Null Then parent=New ISceneNodeDefault
		Local keymap_size:Int=0
		Local keymap:Int[]=GetHandleArray(keyMapArray)
		If keymap Then keymap_size=keymap.Dimensions()[0]
			
		Return ICameraSceneNode.createFromHandle(IrrScene_ISceneManager_addCameraSceneNodeFPS(handle,parent.handle,rotateSpeed,moveSpeed,id,keymap,keymap_size,bNoVerticalMovement,jumpSpeed,bInvertMouse),False)
	EndMethod
	
	Method addCameraSceneNodeMaya:ICameraSceneNode(parent:ISceneNode=Null,rotateSpeed:Float=-1500.0,zoomSpeed:Float=200.0,translationSpeed:Float=1500.0,id:Int=-1)
		If parent=Null Then parent=New ISceneNodeDefault
		Return ICameraSceneNode.createFromHandle(IrrScene_ISceneManager_addCameraSceneNodeMaya(handle,parent.handle,rotateSpeed,zoomSpeed,translationSpeed,id),False)
	EndMethod

	Method addCameraSceneNode:ICameraSceneNode(parent:ISceneNode=Null,pos:Vector3df=Null,lookat:Vector3df=Null,id:Int=-1)
		If parent=Null Then parent=New ISceneNodeDefault
		If pos=Null Then pos=New Vector3df
		If lookat=Null Then lookat=New Vector3df
		Return ICameraSceneNode.createFromHandle(IrrScene_ISceneManager_addCameraSceneNode(handle,parent.handle,pos.handle,lookat.handle,id),False)
	EndMethod

	Method addOctTreeSceneNode:IMeshSceneNode(mesh:IMesh,parent:ISceneNode=Null,id:Int=-1,minimalPolysPerNode:Int=512,bAlsoAddIfMeshPointerZero:Int=False)	
		Return IMeshSceneNode.createFromHandle(IrrScene_ISceneManager_addOctTreeSceneNode(handle,_HANDLE(mesh),_HANDLE(parent),id,minimalPolysPerNode,bAlsoAddIfMeshPointerZero),False)
	EndMethod
	
	Method addAnimatedOctTreeSceneNode:IMeshSceneNode(mesh:IAnimatedMesh,parent:ISceneNode=Null,id:Int=-1,minimalPolysPerNode:Int=128)
		Return IMeshSceneNode.createFromHandle(IrrScene_ISceneManager_addAnimatedOctTreeSceneNode(handle,_HANDLE(mesh),_HANDLE(parent),id,minimalPolysPerNode),False)
	EndMethod
	
	Method addWaterSurfaceSceneNode:ISceneNode(mesh:IMesh,waveHeight:Float=2.0,waveSpeed:Float=300.0,waveLength:Float=10.0,parent:ISceneNode=Null,id:Int=-1,pos:Vector3df=Null,rot:Vector3df=Null,scale:Vector3df=Null)
		If parent=Null Then parent=New ISceneNodeDefault
		If pos=Null Then pos=New Vector3df
		If rot=Null Then rot=New Vector3df
		If scale=Null Then scale=New Vector3df
		Return ISceneNode.createFromHandle(IrrScene_ISceneManager_addWaterSurfaceSceneNode(handle,mesh.handle,waveHeight,waveSpeed,waveLength,parent.handle,id,pos.handle,rot.handle,scale.handle),False)
	EndMethod

	Method addMeshSceneNode:IMeshSceneNode(mesh:IMesh,parent:ISceneNode=Null,id:Int=-1,pos:Vector3df=Null,rot:Vector3df=Null,scale:Vector3df=Null,bAlsoAddIfMeshPointerZero:Int=False)
		If parent=Null Then parent=New ISceneNodeDefault
		If pos=Null Then pos=New Vector3df
		If rot=Null Then rot=New Vector3df
		If scale=Null Then scale=New Vector3df
		Return IMeshSceneNode.createFromHandle(IrrScene_ISceneManager_addMeshSceneNode(handle,mesh.handle,parent.handle,id,pos.handle,rot.handle,scale.handle,bAlsoAddIfMeshPointerZero),False)
	EndMethod

	Method addAnimatedMeshSceneNode:IAnimatedMeshSceneNode(mesh:IAnimatedMesh,parent:ISceneNode=Null,id:Int=-1,pos:Vector3df=Null,rot:Vector3df=Null,scale:Vector3df=Null,bAlsoAddIfMeshPointerZero:Int=False)
		If parent=Null Then parent=New ISceneNodeDefault
		If pos=Null Then pos=New Vector3df
		If rot=Null Then rot=New Vector3df
		If scale=Null Then scale=New Vector3df
		Return IAnimatedMeshSceneNode.createFromHandle(IrrScene_ISceneManager_addAnimatedMeshSceneNode(handle,mesh.handle,parent.handle,id,pos.handle,rot.handle,scale.handle,bAlsoAddIfMeshPointerZero),False)
	EndMethod

	Method addCubeSceneNode:IMeshSceneNode(size:Float=10.0,parent:ISceneNode=Null,id:Int=-1,pos:Vector3df=Null,rot:Vector3df=Null,scale:Vector3df=Null)
		If parent=Null Then parent=New ISceneNodeDefault
		If pos=Null Then pos=New Vector3df
		If rot=Null Then rot=New Vector3df
		If scale=Null Then scale=New Vector3df
		Return IMeshSceneNode.createFromHandle(IrrScene_ISceneManager_addCubeSceneNode(handle,size,parent.handle,id,pos.handle,rot.handle,scale.handle),False)
	EndMethod
	
	Method addSphereSceneNode:IMeshSceneNode(radius:Float=5.0,polyCount:Int=16,parent:ISceneNode=Null,id:Int=-1,pos:Vector3df=Null,rot:Vector3df=Null,scale:Vector3df=Null)
		If parent=Null Then parent=New ISceneNodeDefault
		If pos=Null Then pos=New Vector3df
		If rot=Null Then rot=New Vector3df
		If scale=Null Then scale=New Vector3df
		Return IMeshSceneNode.createFromHandle(IrrScene_ISceneManager_addSphereSceneNode(handle,radius,polyCount,parent.handle,id,pos.handle,rot.handle,scale.handle),False)
	EndMethod
	
	Method getVideoDriver:IVideoDriver()
		Return IVideoDriver.createFromHandle(IrrScene_ISceneManager_getVideoDriver:Int(handle), False)
	EndMethod
	
	Method getMesh:IAnimatedMesh(filename:String)
		Return IAnimatedMesh.createFromHandle(IrrScene_ISceneManager_getMesh(handle,filename), False)
	EndMethod

	Method getMeshFromFileIO:IAnimatedMesh(file:IReadFile)
		Return IAnimatedMesh.createFromHandle(IrrScene_ISceneManager_getMeshFromFileIO(handle, file.handle), False)
	EndMethod
	
	Method drawAll()
		IrrScene_ISceneManager_drawAll(handle)
	EndMethod

	Method getMeshCache:IMeshCache()
		Return IMeshCache.createFromHandle(IrrScene_ISceneManager_getMeshCache(handle),False)
	EndMethod

	Method createNewSceneManager:ISceneManager(bCloneContent:Int=False)
		Return ISceneManager.createFromHandle(IrrScene_ISceneManager_createNewSceneManager(handle,bCloneContent),True)
	EndMethod

	Method addSkyDomeSceneNode:ISceneNode(texture:ITexture,horiRes:Int = 16, vertRes:Int = 8, texturePercentage:Float = 0.9, spherePercentage:Float = 2.0, radius:Float = 1000.0, parent:ISceneNode=Null,id:Int=-1)
		If parent=Null Then parent=New ISceneNodeDefault
		Return ISceneNode.createFromHandle(IrrScene_ISceneManager_addSkyDomeSceneNode(handle,texture.handle,horiRes,vertRes,texturePercentage,spherePercentage,radius,parent.handle,id),False)
	EndMethod

	Method getDefaultSceneNodeFactory:ISceneNodeFactory()
		Return ISceneNodeFactory.createFromHandle(IrrScene_ISceneManager_getDefaultSceneNodeFactory(handle),False)
	EndMethod

	Method registerSceneNodeFactory(factoryToAdd:ISceneNodeFactory)
		IrrScene_ISceneManager_registerSceneNodeFactory(handle,factoryToAdd.handle)
	EndMethod

	Method getRegisteredSceneNodeFactoryCount:Int()
		Return IrrScene_ISceneManager_getRegisteredSceneNodeFactoryCount(handle)
	EndMethod
	
	Method getSceneNodeFactory:ISceneNodeFactory(index:Int)
		Return ISceneNodeFactory.createFromHandle(IrrScene_ISceneManager_getSceneNodeFactory(handle,index),False)
	EndMethod
	
	Method getDefaultSceneNodeAnimatorFactory:ISceneNodeAnimatorFactory()
		Return ISceneNodeAnimatorFactory.createFromHandle(IrrScene_ISceneManager_getDefaultSceneNodeAnimatorFactory(handle),False)
	EndMethod
	
	Method registerSceneNodeAnimatorFactory(factoryToAdd:ISceneNodeAnimatorFactory)
		IrrScene_ISceneManager_registerSceneNodeAnimatorFactory(handle,factoryToAdd.handle)
	EndMethod
	
	Method getRegisteredSceneNodeAnimatorFactoryCount:Int()
		Return IrrScene_ISceneManager_getRegisteredSceneNodeAnimatorFactoryCount(handle)
	EndMethod
	
	Method getSceneNodeAnimatorFactory:ISceneNodeAnimatorFactory(index:Int)
		Return ISceneNodeAnimatorFactory.createFromHandle(IrrScene_ISceneManager_getSceneNodeAnimatorFactory(handle,index),False)
	EndMethod

	Method getParameters:IAttributes()
		Return IAttributes.createFromHandle(IrrScene_ISceneManager_getParameters(handle),False)
	EndMethod

	Method getSceneNodeTypeName:String(ESCENE_NODE_TYPE:Int)
		Return IrrScene_ISceneManager_getSceneNodeTypeName(handle,ESCENE_NODE_TYPE)
	EndMethod

	Method saveScene:Int(filename:String,userDataSerializer:ISceneUserDataSerializer=Null)
		Return saveSceneToFile(filename,userDataSerializer)
	EndMethod
	
	Method saveSceneToFile:Int(filename:String,userDataSerializer:ISceneUserDataSerializer=Null)
		Local UDhandle:Int=0
		If userDataSerializer And userDataSerializer.IsValid() Then UDhandle=userDataSerializer.handle
		Return IrrScene_ISceneManager_saveSceneToFile(handle,filename,UDhandle)
	EndMethod
	
	Method saveSceneToFileIO:Int(file:IWriteFile,userDataSerializer:ISceneUserDataSerializer=Null)
		Local UDhandle:Int=0
		If userDataSerializer And userDataSerializer.IsValid() Then UDhandle=userDataSerializer.handle
		Return IrrScene_ISceneManager_saveSceneToFileIO(handle,file.handle,UDhandle)
	EndMethod

	Method loadScene:Int(filename:String,userDataSerializer:ISceneUserDataSerializer=Null)
		Return loadSceneFromFile(filename,userDataSerializer)
	EndMethod
	
	Method loadSceneFromFile:Int(filename:String,userDataSerializer:ISceneUserDataSerializer=Null)
		Local UDhandle:Int=0
		If userDataSerializer And userDataSerializer.IsValid() Then UDhandle=userDataSerializer.handle
		Return IrrScene_ISceneManager_loadSceneFromFile(handle,filename,UDhandle)
	EndMethod
	
	Method loadSceneFromFileIO:Int(file:IReadFile,userDataSerializer:ISceneUserDataSerializer=Null)
		Local UDhandle:Int=0
		If userDataSerializer And userDataSerializer.IsValid() Then UDhandle=userDataSerializer.handle
		Return IrrScene_ISceneManager_loadSceneFromFileIO(handle,file.handle,UDhandle)
	EndMethod

	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrScene_ISceneManager_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrScene_ISceneManager_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrScene_ISceneManager_grab(handle)
	EndMethod

	Method Delete()
		If (mustdelete) Then drop()
	EndMethod

	Method setAmbientLight(ambientColor:SColorf)
		IrrScene_ISceneManager_setAmbientLight(handle,ambientColor.handle)
	EndMethod
	
	Method getAmbientLight:SColorf()
		Return SColorf.createFromHandle(IrrScene_ISceneManager_getAmbientLight(handle),True)
	EndMethod
	
	Method getGUIEnvironment:IGUIEnvironment()
		Return IGUIEnvironment.createFromHandle(IrrScene_ISceneManager_getGUIEnvironment(handle),False)
	EndMethod
	
	Method addBillboardTextSceneNode:IBillboardTextSceneNode(font:IGUIFont,text:String,parent:ISceneNode=Null,size:Dimension2df=Null,pos:Vector3df=Null,id:Int=-1,colorTop:SCOlor=Null,colorBottom:SColor=Null)
		Return IBillboardTextSceneNode.createFromHandle(IrrScene_ISceneManager_addBillboardTextSceneNode(handle, ..
			font.handle, StringToWChar(text), _HANDLE(parent), _HANDLE(size), _HANDLE(pos), id, _HANDLE(colorTop), _HANDLE(colorBottom)), False)
	EndMethod
	
	Method addArrowMesh:IAnimatedMesh(name:String, vtxColor0:SColor = Null,vtxColor1:SColor = Null, ..
		tesselationCylinder:Int = 4, tesselationCone:Int = 8, ..
		height:Float = 1, cylinderHeight:Float = .6, width0:Float = .05, width1:Float = .3)
					
		Return IAnimatedMesh.createFromHandle(IrrScene_ISceneManager_addArrowMesh(handle,name, ..
			_HANDLE(vtxColor0), _HANDLE(vtxColor1), ..
			tesselationCylinder,tesselationCone,height,cylinderHeight,width0,width1),False)
	EndMethod
	
	Method addSphereMesh:IAnimatedMesh(name:String, radius:Float = 5.0, polyCountX:Int = 16, polyCountY:Int = 16)
		Return IAnimatedMesh.createFromHandle(IrrScene_ISceneManager_addSphereMesh(handle, name, radius, polyCountX, polyCountY), False)
	EndMethod
	
	Method addQuake3SceneNode:IMeshSceneNode(meshBuffer:IMeshBuffer,q3shader:IShader,parent:ISceneNode=Null,id:Int=-1)
		Return IMeshSceneNode.createFromHandle(IrrScene_ISceneManager_addQuake3SceneNode(handle,_HANDLE(meshBuffer),_HANDLE(q3shader),_HANDLE(parent),id),False)
	EndMethod

	Method getSceneNodeFromType:ISceneNode(ESCENE_NODE_TYPE:Int,start:ISceneNode = Null)
		Return ISceneNode.createFromHandle(IrrScene_ISceneManager_getSceneNodeFromType(handle,ESCENE_NODE_TYPE,_HANDLE(start)),False)
	EndMethod
	
	Method addSceneNode:ISceneNode(sceneNodeTypeName:String,parent:ISceneNode = Null)
		Return ISceneNode.createFromHandle(IrrScene_ISceneManager_addSceneNode(handle,sceneNodeTypeName,_HANDLE(parent)),False)
	EndMethod
	
	Method getSceneNodesFromType:ISceneNode[](ESCENE_NODE_TYPE:Int,start:ISceneNode=Null)
		Local ptrs:Int Ptr	
		Local objs:ISceneNode[] = Null				

		Local size:Int = IrrScene_ISceneManager_getSceneNodesFromType(handle,ESCENE_NODE_TYPE,Varptr(ptrs),_HANDLE(start))

		If size>0 Then		
			objs=New ISceneNode[size]

			For Local i:Int=0 Until size
				objs[i]=ISceneNode.createFromHandle(ptrs[i],False)
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		Return objs	
	EndMethod

	Method addVolumeLightSceneNode:IVolumeLightSceneNode(parent:ISceneNode, ..
		id:Int = -1, subdivU:Int = 32, subdivV:Int = 32, ..
		foot:SColor = Null, tail:SColor = Null, ..
		position:Vector3df = Null, rotation:Vector3df = Null, scale:Vector3df = Null)
		
		Return IVolumeLightSceneNode.createFromHandle( ..
			IrrScene_ISceneManager_addVolumeLightSceneNode(handle, ..
				_HANDLE(parent), id, subdivU, subdivV, ..
				_HANDLE(foot), _HANDLE(tail), ..
				_HANDLE(position), _HANDLE(rotation), _HANDLE(scale)), False)
	EndMethod
	
	Method setLightManager(lightManager:ILightManager)
		IrrScene_ISceneManager_setLightManager(handle, _HANDLE(lightManager))
	EndMethod

	Method addVolumeLightMesh:IAnimatedMesh(name:String, SubdivideU:Int = 32, SubdivideV:Int = 32, FootColor:SColor = Null, TailColor:SColor = Null)
		Return IAnimatedMesh.createFromHandle(IrrScene_ISceneManager_addVolumeLightMesh(handle, name, SubdivideU, SubdivideV, _HANDLE(FootColor), _HANDLE(TailColor)), False)
	EndMethod
	
	Method getAnimatorTypeName:String(ESCENE_NODE_ANIMATOR_TYPE:Int)
		Return IrrScene_ISceneManager_getAnimatorTypeName(handle, ESCENE_NODE_ANIMATOR_TYPE)
	EndMethod
	
	Method creatSkinnedMesh:ISkinnedMesh()
		Return ISkinnedMesh.createFromHandle(IrrScene_ISceneManager_createSkinnedMesh(handle), True)
	EndMethod
	
	Method getGeometryCreator:IGeometryCreator()
		Return IGeometryCreator.createFromHandle(IrrScene_ISceneManager_getGeometryCreator(handle), False)
	EndMethod
	
	Method isCulled:Int(node:ISceneNode)
		Return IrrScene_ISceneManager_isCulled(handle, _HANDLE(node))
	EndMethod

EndType


