' ------------------------------------------------------------------------------------
'  irr::scene::ITriangleSelector wrappers
' ------------------------------------------------------------------------------------
Public

Type ITriangleSelector Extends IReferenceCounted

	Function createFromHandle:ITriangleSelector(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:ITriangleSelector=New ITriangleSelector
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction
	
	Method getTriangleCount:Int()
		Return IrrScene_ITriangleSelector_getTriangleCount(handle)
	EndMethod
	
	Method getTriangles:Triangle3df[](transform:Matrix4=Null)
		If transform=Null Then transform=New Matrix4 ' default the handle to 0

		Local ptrs:Int[] = New Int[getTriangleCount()]
		Local objs:Triangle3df[]=Null

		Local size:Int=IrrScene_ITriangleSelector_getTriangles(handle,ptrs,transform.handle)
		
		If size>0 Then		
			objs=New Triangle3df[size]

			For Local i:Int=0 Until size
				objs[i]=Triangle3df.createFromHandle(ptrs[i],True)
			Next
		EndIf
				
		Return objs
	EndMethod
	
	Method getTrianglesOnLine:Triangle3df[](line:Line3df,transform:Matrix4=Null)
		If transform=Null Then transform=New Matrix4 ' default the handle to 0

		Local ptrs:Int[] = New Int[getTriangleCount()]
		Local objs:Triangle3df[]=Null

		Local size:Int = IrrScene_ITriangleSelector_getTrianglesOnLine(handle,ptrs,line.handle,transform.handle)

		If size>0 Then		
			objs=New Triangle3df[size]

			For Local i:Int=0 Until size
				objs[i]=Triangle3df.createFromHandle(ptrs[i],True)
			Next
		EndIf
				
		Return objs
	EndMethod
	
	Method getTrianglesInBox:Triangle3df[](box:AABbox3df,transform:Matrix4=Null)
		If transform=Null Then transform=New Matrix4 ' default the handle to 0

		Local ptrs:Int[] = New Int[getTriangleCount()]
		Local objs:Triangle3df[]=Null

		Local size:Int = IrrScene_ITriangleSelector_getTrianglesInBox(handle,ptrs,box.handle,transform.handle)

		If size>0 Then		
			objs=New Triangle3df[size]

			For Local i:Int=0 Until size
				objs[i]=Triangle3df.createFromHandle(ptrs[i],True)
			Next
		EndIf
				
		Return objs
	EndMethod

	Method getSceneNodeForTriangle:ISceneNode(triangleIndex:Int)
		Return ISceneNode.createFromHandle(IrrScene_ITriangleSelector_getSceneNodeForTriangle(handle, triangleIndex), False)
	EndMethod

	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrScene_ITriangleSelector_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrScene_ITriangleSelector_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrScene_ITriangleSelector_grab(handle)
	EndMethod

	Method Delete()
		If (mustdelete) Then drop()
	EndMethod
EndType

