' ------------------------------------------------------------------------------------
' irr::video::IVideoDriver wrappers
' use irr:IrrlichtDevice.getVideoDriver() To get a pointer For this
' ------------------------------------------------------------------------------------

Public

Type IVideoDriver Extends IReferenceCounted Final

	Function createFromHandle:IVideoDriver(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:IVideoDriver=New IVideoDriver
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Method beginScene:Int(bBackBuffer:Int, bZBuffer:Int, color:SColor = Null, windowId:Byte Ptr = Null, sourceRect:Recti = Null)
		Return IrrVideo_IVideoDriver_beginScene(handle,bBackBuffer,bZBuffer,_HANDLE(color), windowId, _HANDLE(sourceRect))
	EndMethod
	
	Method endScene:Int()
		Return IrrVideo_IVideoDriver_endScene(handle)
	EndMethod
	
	Method queryFeature:Int(E_VIDEO_DRIVER_FEATURE:Int)
		Return IrrVideo_IVideoDriver_queryFeature(handle,E_VIDEO_DRIVER_FEATURE)
	EndMethod
	
	Method getGPUProgrammingServices:IGPUProgrammingServices()
		Return IGPUProgrammingServices.createFromHandle(IrrVideo_IVideoDriver_getGPUProgrammingServices(handle),False)
	EndMethod

	Method getMeshManipulator:IMeshManipulator()
		Return IMeshManipulator.createFromHandle(IrrVideo_IVideoDriver_getMeshManipulator(handle), False)
	EndMethod

	Method SetTransform(E_TRANSFORMATION_STATE:Int,matrix:Matrix4)
		IrrVideo_IVideoDriver_setTransform(handle,E_TRANSFORMATION_STATE,matrix.handle)
	EndMethod
	
	Method getTransform:Matrix4(E_TRANSFORMATION_STATE:Int)
		Return Matrix4.createFromHandle(IrrVideo_IVideoDriver_getTransform(handle,E_TRANSFORMATION_STATE))
	EndMethod
	
	Method setMaterial(material:SMaterial)
		IrrVideo_IVideoDriver_setMaterial(handle,material.handle)
	EndMethod
	
	Method getTexture:ITexture(filename:String)
		Return ITexture.createFromHandle(IrrVideo_IVideoDriver_getTexture(handle,filename))
	EndMethod	

	Method getTextureFromStream:ITexture(read:IReadFile)
		Return ITexture.createFromHandle(IrrVideo_IVideoDriver_getTextureFromStream(handle,read.handle))
	EndMethod
	
	Method addTexture:ITexture(size:Dimension2di,name:String,ECOLOR_FORMAT:Int=ECF_A8R8G8B8)
		Return ITexture.createFromHandle(IrrVideo_IVideoDriver_addTexture(handle,size.handle,name,ECOLOR_FORMAT))
	EndMethod
	
	Method addTextureFromImage:ITexture(name:String,img:IImage)
		Return ITexture.createFromHandle(IrrVideo_IVideoDriver_addTextureFromImage(handle,name,img.handle))
	EndMethod
	
	Method removeTexture(texture:ITexture)
		IrrVideo_IVideoDriver_removeTexture(handle,texture.handle)
	EndMethod
	
	Method removeAllTextures()
		IrrVideo_IVideoDriver_removeAllTextures(handle)
	EndMethod
	
	Method makeColorKeyTexture(texture:ITexture, color:SColor, zeroTexels:Int = False)
		IrrVideo_IVideoDriver_makeColorKeyTexture(handle,texture.handle,color.handle, zeroTexels)
	EndMethod
	
	Method makeColorKeyTextureFromPixel(texture:ITexture,pos:Position2di, zeroTexels:Int = False)
		IrrVideo_IVideoDriver_makeColorKeyTextureFromPixel(handle,texture.handle,pos.handle, zeroTexels)
	EndMethod

	Method makeNormalMapTexture(texture:ITexture,amplitude:Float=1.0)	
		IrrVideo_IVideoDriver_makeNormalMapTexture(handle,texture.handle,amplitude)
	EndMethod 
	
	Method setRenderTarget(texture:ITexture,bClearBackBuffer:Int=True,bClearZBuffer:Int=True,color:SColor=Null)
		IrrVideo_IVideoDriver_setRenderTarget(handle,_HANDLE(texture),bClearBackBuffer,bClearZBuffer,_HANDLE(color))
	EndMethod

	Method setRenderTargetByEnum(E_RENDER_TARGET:Int, bClearBackBuffer:Int=True, bClearZBuffer:Int=True, color:SColor=Null)
		IrrVideo_IVideoDriver_setRenderTargetByEnum(handle, E_RENDER_TARGET, bClearBackBuffer, bClearZBuffer, _HANDLE(color))
	EndMethod

	Method SetViewport(area:Recti)
		IrrVideo_IVideoDriver_setViewPort(handle,area.handle)
	EndMethod 
	
	Method GetViewport:Recti()
		Return Recti.createFromHandle(IrrVideo_IVideoDriver_getViewPort(handle))
	EndMethod

	Method drawVertexPrimitiveList(vertices:Byte Ptr, vertexCount:Int, indexList:Byte Ptr, primCount:Int, E_VERTEX_TYPE:Int = EVT_STANDARD, E_PRIMITIVE_TYPE:Int = EPT_TRIANGLES, E_INDEX_TYPE:Int = EIT_16BIT)
		IrrVideo_IVideoDriver_drawVertexPrimitiveList(handle, vertices, vertexCount, indexList, primCount, E_VERTEX_TYPE, E_PRIMITIVE_TYPE, E_INDEX_TYPE)
	EndMethod

	Method draw2DVertexPrimitiveList(vertices:Byte Ptr, vertexCount:Int, indexList:Byte Ptr, primCount:Int, E_VERTEX_TYPE:Int = EVT_STANDARD, E_PRIMITIVE_TYPE:Int = EPT_TRIANGLES, E_INDEX_TYPE:Int = EIT_16BIT)
		IrrVideo_IVideoDriver_draw2DVertexPrimitiveList(handle, vertices, vertexCount, indexList, primCount, E_VERTEX_TYPE, E_PRIMITIVE_TYPE, E_INDEX_TYPE)
	EndMethod
	
	Method draw3DLine(start:Vector3df,pend:Vector3df,color:SColor=Null) ' def color=255,255,255,255
		If color=Null Then color=New SColor ' handle will be 0 tripping the default
		IrrVideo_IVideoDriver_draw3DLine(handle,start.handle,pend.handle,color.handle)
	EndMethod

	Method draw3DTriangle(triangle:Triangle3df,color:SColor=Null) ' def color=255,255,255,255
		If color=Null Then color=New SColor ' handle will be 0 tripping the default
		IrrVideo_IVideoDriver_draw3DTriangle(handle,triangle.handle,color.handle) 
	EndMethod
	
	Method draw3DBox(box:AABbox3df,color:SColor=Null) ' def color=255,255,255,255
		If color=Null Then color=New SColor ' handle will be 0 tripping the default
		IrrVideo_IVideoDriver_draw3DBox(handle,box.handle,color.handle)
	EndMethod
	
	Method draw2DImage(texture:ITexture,destPos:Position2di)
		IrrVideo_IVideoDriver_draw2DImage(handle,texture.handle,destPos.handle)
	EndMethod

	Method draw2DImageSegment(texture:ITexture,destPos:Position2di,sourceRect:Recti,clipRect:Recti=Null,color:SColor=Null,bUseAlphaChannelOfTexture:Int=False)
		IrrVideo_IVideoDriver_draw2DImageSegment(handle,texture.handle,destPos.handle,sourceRect.handle,_HANDLE(clipRect),_HANDLE(color),bUseAlphaChannelOfTexture)
	EndMethod

	Method draw2DImageSegmentToRect(texture:ITexture,destRect:Recti,sourceRect:Recti,clipRect:Recti=Null,colors:SColor[]=Null,bUseAlphaChannelOfTexture:Int=False)
		If clipRect=Null Then clipRect=New Recti	' handle will be 0 tripping the default
		Local colors_size:Int=0
		Local colorsArr:Int[]=GetHandleArray(colors)
		If colorsArr Then colors_size=colorsArr.Dimensions()[0]

		IrrVideo_IVideoDriver_draw2DImageSegmentToRect(handle,texture.handle,destRect.handle,sourceRect.handle,clipRect.handle,colorsArr,colors_size,bUseAlphaChannelOfTexture)
	EndMethod

	Method draw2DImageBatch(texture:ITexture, pos:Position2di, sourceRects:Recti[], indices:Int[], kerningWidth:Int = 0, clipRect:Recti = Null, color:SColor = Null, bUseAlphaChannelOfTexture:Int = False)

		Local indices_size:Int = 0
		If indices Then indices_size = indices.Dimensions()[0]

		Local sourceRects_size:Int = 0
		Local sourceRects_array:Int[] = GetHandleArray(sourceRects)
		If sourceRects_array Then sourceRects_size = sourceRects_array.Dimensions()[0]

		IrrVideo_IVideoDriver_draw2DImageBatch(handle, _HANDLE(texture), _HANDLE(pos), sourceRects_array, sourceRects_size, indices, indices_size, kerningWidth, _HANDLE(clipRect), _HANDLE(color), bUseAlphaChannelOfTexture)	
	EndMethod
	

	Method draw2DImageBatchAtPositions(texture:ITexture, positions:Position2di[], sourceRects:Recti[], clipRect:Recti = Null, color:SColor = Null, bUseAlphaChannelOfTexture:Int = False)
		Local pos_size:Int = 0
		Local pos_array:Int[] = GetHandleArray(positions)
		If pos_array Then pos_size = pos_array.Dimensions()[0]

		Local sourceRects_size:Int = 0
		Local sourceRects_array:Int[] = GetHandleArray(sourceRects)
		If sourceRects_array Then sourceRects_size = sourceRects_array.Dimensions()[0]

		IrrVideo_IVideoDriver_draw2DImageBatchAtPositions(handle, _HANDLE(texture), pos_array, pos_size, sourceRects_array, sourceRects_size, _HANDLE(clipRect), _HANDLE(color), bUseAlphaChannelOfTexture) 
	EndMethod
	
	Method draw2DRectangle(color:SColor,pos:Recti,clip:Recti=Null)
		IrrVideo_IVideoDriver_draw2DRectangle(handle,color.handle,pos.handle,_HANDLE(clip))
	EndMethod

	Method draw2DRectangleWithGradient(pos:Recti,colorLeftUp:SColor,colorRightUp:SColor,colorLeftDown:SColor,colorRightDown:SColor,clip:Recti=Null)
		IrrVideo_IVideoDriver_draw2DRectangleWithGradient(handle,pos.handle,colorLeftUp.handle,colorRightUp.handle,colorLeftDown.handle,colorRightDown.handle,_HANDLE(clip))
	EndMethod

	Method draw2DRectangleOutline(pos:Recti, color:SColor = Null)
		IrrVideo_IVideoDriver_draw2DRectangleOutline(handle, _HANDLE(pos), _HANDLE(color))
	EndMethod
	
	Method draw2DLine(start:Position2di,pEnd:Position2di,color:SColor=Null) ' def color=255,255,255,255
		IrrVideo_IVideoDriver_draw2DLine(handle,start.handle,pEnd.handle,_HANDLE(color))
	EndMethod

	Method draw2DPolygon(center:Position2di,radius:Float,color:SColor=Null,vertexCount:Int=10) ' def color=100,255,255,255
		IrrVideo_IVideoDriver_draw2DPolygon(handle,center.handle,radius,_HANDLE(color),vertexCount)
	EndMethod

	Method drawStencilShadowVolume(triangles:Array_triangle3df,count:Int,bZfail:Int=True)
		IrrVideo_IVideoDriver_drawStencilShadowVolume(handle,triangles.handle,count,bZfail)
	EndMethod
	
	Method drawStencilShadow(bClearStencilBuffer:Int=False,leftUpEdge:SColor=Null,rightUpEdge:SColor=Null,leftDownEdge:SColor=Null,rightDownEdge:SColor=Null) ' colors default to 255,0,0,0
		IrrVideo_IVideoDriver_drawStencilShadow(handle,bClearStencilBuffer,_HANDLE(leftUpEdge),_HANDLE(rightUpEdge),_HANDLE(leftDownEdge),_HANDLE(rightDownEdge))
	EndMethod

	Method drawMeshBuffer(mb:IMeshBuffer)
		IrrVideo_IVideoDriver_drawMeshBuffer(handle,mb.handle)
	EndMethod
	
	Method setFog(color:SColor=Null,E_FOG_TYPE:Int = EFT_FOG_LINEAR,start:Float=50.0,fend:Float=100.0,density:Float=0.01,bPixelFog:Int=False,bRangeFog:Int=False) ' color defaults to 0,255,255,255
		IrrVideo_IVideoDriver_setFog(handle,_HANDLE(color),E_FOG_TYPE,start,fend,density,bPixelFog,bRangeFog)
	EndMethod
	
	Method getScreenSize:Dimension2di()
		Return Dimension2di.createFromHandle(IrrVideo_IVideoDriver_getScreenSize(handle))
	EndMethod
	
	Method getCurrentRenderTargetSize:Dimension2di()
		Return Dimension2di.createFromHandle(IrrVideo_IVideoDriver_getCurrentRenderTargetSize(handle),True)
	EndMethod

	Method getFPS:Int()
		Return IrrVideo_IVideoDriver_getFPS(handle)
	EndMethod
	
	Method getPrimitiveCountDrawn:Int(param:Int=0) ' returns unsigned 32
		Return IrrVideo_IVideoDriver_getPrimitiveCountDrawn(handle,param) 
	EndMethod
	
	Method deleteAllDynamicLights()
		IrrVideo_IVideoDriver_deleteAllDynamicLights(handle)
	EndMethod
	
	Method addDynamicLight(light:SLight)
		IrrVideo_IVideoDriver_addDynamicLight(handle,light.handle)
	EndMethod
	
	Method setAmbientLight(color:SColorf)
		IrrVideo_IVideoDriver_setAmbientLight(handle,color.handle)
	EndMethod
	
	Method getMaximalDynamicLightAmount:Int()
		Return IrrVideo_IVideoDriver_getMaximalDynamicLightAmount(handle)
	EndMethod
	
	Method getDynamicLightCount:Int()
		Return IrrVideo_IVideoDriver_getDynamicLightCount(handle)
	EndMethod
	
	Method getDynamicLight:SLight(idx:Int)
		Return SLight.createFromHandle(IrrVideo_IVideoDriver_getDynamicLight(handle,idx))
	EndMethod
	
	Method getName:String()
		 Return IrrVideo_IVideoDriver_getName(handle)
	EndMethod
	
	Method getMaximalPrimitiveCount:Int()
		Return IrrVideo_IVideoDriver_getMaximalPrimitiveCount(handle)
	EndMethod
	
	Method setTextureCreationFlag(E_TEXTURE_CREATION_FLAG:Int,bEnabled:Int = True)
		IrrVideo_IVideoDriver_setTextureCreationFlag(handle,E_TEXTURE_CREATION_FLAG,bEnabled)
	EndMethod
	
	Method getTextureCreationFlag:Int(E_TEXTURE_CREATION_FLAG:Int)
		Return IrrVideo_IVideoDriver_getTextureCreationFlag(handle,E_TEXTURE_CREATION_FLAG)
	EndMethod
	
	Method createImageFromFile:IImage(filename:String)
		Return IImage.createFromHandle(IrrVideo_IVideoDriver_createImageFromFile(handle,filename))
	EndMethod
	
	Method createImageFromStream:IImage(read:IReadFile)
		Return IImage.createFromHandle(IrrVideo_IVideoDriver_createImageFromStream(handle,read.handle))
	EndMethod
	
	Method createImageFromData:IImage(ECOLOR_FORMAT:Int,size:Dimension2di,pdata:Byte Ptr,ownForeignMemory:Int=False,deleteMemory:Int=True)
		Return IImage.createFromHandle(IrrVideo_IVideoDriver_createImageFromData(handle,ECOLOR_FORMAT,size.handle,pdata,ownForeignMemory,deleteMemory))
	EndMethod

	Method createImageEmpty:IImage(ECOLOR_FORMAT:Int, size:Dimension2di)
		Return IImage.createFromHandle(IrrVideo_IVideoDriver_createImageEmpty(handle, ECOLOR_FORMAT, _HANDLE(size)), True)
	EndMethod
	
	Method createImageFromImage:IImage(ECOLOR_FORMAT:Int, imageToCopy:IImage)
		Return IImage.createFromHandle(IrrVideo_IVideoDriver_createImageFromImage(handle, ECOLOR_FORMAT, _HANDLE(imageToCopy)), True)
	EndMethod
	
	Method createImageFromImagePart:IImage(imageToCopy:IImage, pos:Position2di, size:Dimension2di)
		Return IImage.createFromHandle(IrrVideo_IVideoDriver_createImageFromImagePart(handle, _HANDLE(imageToCopy), _HANDLE(pos), _HANDLE(size)), True)
	EndMethod

	Method createImageFromTexturePart:IImage(texture:ITexture, pos:Position2di, size:Dimension2di)
		Return IImage.createFromHandle(IrrVideo_IVideoDriver_createImageFromTexturePart(handle, _HANDLE(texture), _HANDLE(pos), _HANDLE(size)))
	EndMethod

	Method OnResize(size:Dimension2di)
		IrrVideo_IVideoDriver_OnResize(handle, _HANDLE(size))
	EndMethod

	Method getMaterialRendererCount:Int()
		Return IrrVideo_IVideoDriver_getMaterialRendererCount(handle)
	EndMethod
	
	' dont Delete reference
	Method getMaterialRenderer:IMaterialRenderer(idx:Int)
		Return IMaterialRenderer.createFromHandle(IrrVideo_IVideoDriver_getMaterialRenderer(handle,idx),False)
	EndMethod 
	
	Method getExposedVideoData:SExposedVideoData()
		Return SExposedVideoData.createFromHandle(IrrVideo_IVideoDriver_getExposedVideoData(handle))
	EndMethod

	Method getDriverType:Int() ' returns E_DRIVER_TYPE	
		Return IrrVideo_IVideoDriver_getDriverType(handle) 
	EndMethod
	
	Method drawIndexedTriangleList(vertices:Array_S3DVertex,indexList:Array_u16)
		IrrVideo_IVideoDriver_drawIndexedTriangleList(handle,vertices.handle,indexList.handle)
	EndMethod 

	Method drawIndexedTriangleListFromCoords(vertices:Array_S3DVertex2TCoords,indexList:Array_u16)
		IrrVideo_IVideoDriver_drawIndexedTriangleListFromCoords(handle,vertices.handle,indexList.handle)
	EndMethod
	
	Method drawIndexedTriangleListTangents(vertices:Array_S3DVertexTangents,indexList:Array_u16)
		IrrVideo_IVideoDriver_drawIndexedTriangleListTangents(handle,vertices.handle,indexList.handle)
	EndMethod

	Method drawIndexedTriangleFan(vertices:Array_S3DVertex,indexList:Array_u16)
		IrrVideo_IVideoDriver_drawIndexedTriangleFan(handle,vertices.handle,indexList.handle)
	EndMethod 

	Method drawIndexedTriangleFanFromCoords(vertices:Array_S3DVertex2TCoords,indexList:Array_u16)
		IrrVideo_IVideoDriver_drawIndexedTriangleFanFromCoords(handle,vertices.handle,indexList.handle)
	EndMethod 
	
	Method clearZBuffer()
		IrrVideo_IVideoDriver_clearZBuffer(handle)
	EndMethod
	
	Method getTextureByIndex:ITexture(index:Int)
		Return ITexture.createFromHandle(IrrVideo_IVideoDriver_getTextureByIndex(handle,index),False)
	EndMethod
	
	Method getTextureCount:Int()
		Return IrrVideo_IVideoDriver_getTextureCount(handle)
	EndMethod

	Method addMaterialRenderer:Int(renderer:IMaterialRenderer,name:String=Null)
		Return IrrVideo_IVideoDriver_addMaterialRenderer(handle,renderer.handle,name)
	EndMethod

	Method addExternalImageLoader(loader:IImageLoader)
		IrrVideo_IVideoDriver_addExternalImageLoader(handle,loader.handle)
	EndMethod
	
	Method addExternalImageWriter(writer:IImageWriter)
		IrrVideo_IVideoDriver_addExternalImageWriter(handle,writer.handle)
	EndMethod
	
	Method writeImageToFile:Int(image:IImage,filename:String,param:Int=0)
		Return IrrVideo_IVideoDriver_writeImageToFile(handle,image.handle,filename,param)
	EndMethod
	
	Method writeImageToFileIO:Int(image:IImage, file:IWriteFile, param:Int = 0)
		Return IrrVideo_IVideoDriver_writeImageToFileIO(handle, _HANDLE(image), _HANDLE(file), param)
	EndMethod
	
	Method getMaterialRendererName:String(idx:Int)
		Return IrrVideo_IVideoDriver_getMaterialRendererName(handle,idx)
	EndMethod
	
	Method setMaterialRendererName(idx:Int,name:String)
		IrrVideo_IVideoDriver_setMaterialRendererName(handle,idx,name)
	EndMethod
	
	Method createAttributesFromMaterial:IAttributes(material:SMaterial)
		Return IAttributes.createFromHandle(IrrVideo_IVideoDriver_createAttributesFromMaterial(handle,material.handle),True)
	EndMethod

	Method fillMaterialStructureFromAttributes(out:SMaterial,attributes:IAttributes)
		IrrVideo_IVideoDriver_fillMaterialStructureFromAttributes(handle,out.handle,attributes.handle)
	EndMethod
	
	Method createScreenShot:IImage()
		Return IImage.createFromHandle(IrrVideo_IVideoDriver_createScreenShot(handle),True)
	EndMethod

	Method renameTexture(texture:ITexture,newName:String)
		IrrVideo_IVideoDriver_renameTexture(handle,texture.handle,newName)
	EndMethod

	Method findTexture:ITexture(filename:String)
		Return ITexture.createFromHandle(IrrVideo_IVideoDriver_findTexture(handle, filename), False)
	EndMethod
	
	Method setClipPlane:Int(index:Int, plane:Plane3df, bEnable:Int = False)
		Return IrrVideo_IVideoDriver_setClipPlane(handle, index, plane.handle, bEnable)
	EndMethod
	
	Method enableClipPlane(index:Int, bEnable:Int)
		IrrVideo_IVideoDriver_enableClipPlane(handle, index, bEnable)
	EndMethod

	Method setAllowZWriteOnTransparent(bFlag:Int)
		IrrVideo_IVideoDriver_setAllowZWriteOnTransparent(handle, bFlag)
	EndMethod

	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrVideo_IVideoDriver_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrVideo_IVideoDriver_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrVideo_IVideoDriver_grab(handle)
	EndMethod
	
	Method Delete()
		If (mustdelete) Then drop()
	EndMethod	
	
	Method disableFeature(E_VIDEO_DRIVER_FEATURE:Int, bFlag:Int = True)
		IrrVideo_IVideoDriver_disableFeature(handle, E_VIDEO_DRIVER_FEATURE, bFlag)
	EndMethod
	
	Method checkDriverReset:Int()
		Return IrrVideo_IVideoDriver_checkDriverReset(handle)
	EndMethod

	Method addRenderTargetTexture:ITexture(size:Dimension2di, name:String = Null)
		Return ITexture.createFromHandle(IrrVideo_IVideoDriver_addRenderTargetTexture(handle, size.handle, name), False)
	EndMethod
	
	Method removeHardwareBuffer(mb:IMeshBuffer)
		IrrVideo_IVideoDriver_removeHardwareBuffer(handle, mb.handle)
	EndMethod
	
	Method removeAllHardwareBuffers()
		IrrVideo_IVideoDriver_removeAllHardwareBuffers(handle)
	EndMethod
	
	Method drawPixel(x:Int, y:Int, color:SColor)
		IrrVideo_IVideoDriver_drawPixel(handle, x, y, color.handle)
	EndMethod
	
	Method getColorFormat:Int() ' returns ECOLOR_FORMAT
		Return IrrVideo_IVideoDriver_getColorFormat(handle) 
	EndMethod
	
	Method getVendorInfo:String()
		Return IrrVideo_IVideoDriver_getVendorInfo(handle) 
	EndMethod

	Method getOverrideMaterial:SOverrideMaterial()
		Return SOverrideMaterial.createFromHandle(IrrVideo_IVideoDriver_getOverrideMaterial(handle), False)
	EndMethod
	
	Method setMinHardwareBufferVertexCount(count:Int)
		IrrVideo_IVideoDriver_setMinHardwareBufferVertexCount(handle, count)
	EndMethod
		
	Method turnLightOn(lightIndex:Int, bTurnOn:Int)
		IrrVideo_IVideoDriver_turnLightOn(handle, lightIndex, bTurnOn)
	EndMethod

	Method getImageLoaderCount:Int()
		Return IrrVideo_IVideoDriver_getImageLoaderCount(handle)
	EndMethod
	
	Method getImageLoader:IImageLoader(n:Int)
		Return IImageLoader.createFromHandle(IrrVideo_IVideoDriver_getImageLoader(handle, n), False)
	EndMethod
	
	Method getImageWriterCount:Int()
		Return IrrVideo_IVideoDriver_getImageWriterCount(handle)
	EndMethod
	
	Method getImageWriter:IImageWriter(n:Int)
		Return IImageWriter.createFromHandle(IrrVideo_IVideoDriver_getImageWriter(handle, n), False)
	EndMethod
	
EndType

Type SOverrideMaterial Extends IrrBase Final

	Method setMaterial(material:SMaterial)
		IrrVideo_SOverrideMaterial_setMaterial(handle, _HANDLE(material))
	EndMethod
	
	Method getMaterial:SMaterial()
		Return SMaterial.createFromHandle(IrrVideo_SOverrideMaterial_getMaterial(handle), False)
	EndMethod
	
	Method getEnableFlags:Int()
		REturn IrrVideo_SOverrideMaterial_getEnableFlags(handle)
	EndMethod
	
	Method setEnableFlags(enable_flags:Int)
		IrrVideo_SOverrideMaterial_setEnableFlags(handle, enable_flags)
	EndMethod
	
	Method getEnablePasses:Short()
		Return IrrVideo_SOverrideMaterial_getEnablePasses(handle)
	EndMethod
	
	Method setEnablePasses(passes:Short)
		IrrVideo_SOverrideMaterial_setEnablePasses(handle, passes)
	EndMethod
	
	Method getEnabled:Int()
		Return IrrVideo_SOverrideMaterial_getEnabled(handle)
	EndMethod
	
	Method setEnabled(bEnabled:Int)
		IrrVideo_SOverrideMaterial_setEnabled(handle, bEnabled)
	EndMethod

	Function createFromHandle:SOverrideMaterial(pHandle:Int, bMustDel:Int=True)
		If Not pHandle Then Return Null
		Local oRetVal:SOverrideMaterial = New SOverrideMaterial
		oRetVal.attach(pHandle, bMustDel)
		Return oRetVal
	EndFunction

EndType