Rem
* Example 002 Quake3Map

This Tutorial shows how To Load a Quake 3 map into the engine, Create a
SceneNode For optimizing the speed of rendering, And how To Create a user
controlled camera.

Please note that you should know the basics of the engine before starting this
tutorial. Just take a Short look at the first tutorial, If you haven't done
this yet: http://irrlicht.sourceforge.net/tut001.html 

Lets start like the HelloWorld example: We Include
the irrlicht header files And an additional file To be able
To ask the user For a driver Type using the console.
EndRem
SuperStrict
Framework Irrlicht.Core

Rem
Like in the HelloWorld example, we Create an IrrlichtDevice with
createDevice(). The difference now is that we ask the user To Select
which video driver To use. The Software device might be
too slow To draw a huge Quake 3 map, but just For the fun of it, we make
this decision possible, too.
EndRem

' ask user For driver

Local driverType:Int=EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

' create device And Exit If creation failed

Local device:IrrlichtDevice = IrrlichtDevice.Create(driverType, _DIMENSION2DI(640, 480))

If Not device Return ' could Not create selected driver.

Rem
Get a pointer To the video driver And the SceneManager so that
we do Not always have To write device.getVideoDriver() And
device.getSceneManager().
EndRem
Local driver:IVideoDriver = device.getVideoDriver()
Local smgr:ISceneManager = device.getSceneManager()

Rem
To display the Quake 3 map, we first need To Load it. Quake 3 maps
are packed into .pk3 files which are nothing Else than .zip files.
So we add the .pk3 file To our irr::io::IFileSystem. After it was added,
we are able To read from the files in that archive as If they are
directly stored on the disk.
EndRem
device.getFileSystem().addZipFileArchive("../media/map-20kdm2.pk3")

Rem
Now we can Load the mesh by calling
irr::scene::ISceneManager::getMesh(). We get a pointer returned To an
irr::scene::IAnimatedMesh. As you might know, Quake 3 maps are Not
really animated, they are only a huge chunk of static geometry with
some materials attached. Hence the IAnimatedMesh consists of only one
frame, so we get the "first frame" of the "animation", which is our
quake level And Create an OctTree scene node with it, using
irr::scene::ISceneManager::addOctTreeSceneNode().
The OctTree optimizes the scene a little bit, trying To draw only geometry
which is currently visible. An alternative To the OctTree would be a
irr::scene::IMeshSceneNode, which would always draw the complete
geometry of the mesh, without optimization. Try it: Use
irr::scene::ISceneManager::addMeshSceneNode() instead of
addOctTreeSceneNode() And compare the primitives drawn by the video
driver. (There is a irr::video::IVideoDriver::getPrimitiveCountDrawn()
Method in the irr::video::IVideoDriver class). Note that this
optimization with the OctTree is only useful when drawing huge meshes
consisting of lots of geometry.
EndRem
Local mesh:IAnimatedMesh = smgr.getMesh("20kdm2.bsp")
Local node:ISceneNode = Null

If mesh Then node = smgr.addOctTreeSceneNode(mesh.getMesh(0), Null, -1, 1024)

Rem
Because the level was Not modelled around the origin (0,0,0), we
translate the whole level a little bit. This is done on
irr::scene::ISceneNode level using the methods
irr::scene::ISceneNode::setPosition() (in this Case),
irr::scene::ISceneNode::SetRotation(), And
irr::scene::ISceneNode::SetScale().
EndRem
If node Then node.setPosition(_VECTOR3DF(-1300,-144,-1249))	

Rem
Now we only need a camera To look at the Quake 3 map.
We want To Create a user controlled camera. There are some
cameras available in the Irrlicht engine. For example the
MayaCamera which can be controlled like the camera in Maya:
Rotate with Left mouse button pressed, Zoom with both buttons pressed,
translate with Right mouse button pressed. This could be created with
irr::scene::ISceneManager::addCameraSceneNodeMaya(). But For this
example, we want To Create a camera which behaves like the ones in
first person shooter games (FPS) And hence use
irr::scene::ISceneManager::addCameraSceneNodeFPS().
EndRem
smgr.addCameraSceneNodeFPS()

Rem
The mouse cursor needs Not be visible, so we hide it via the
irr::IrrlichtDevice::ICursorControl.
EndRem
device.getCursorControl().setVisible(False)

Rem
We have done everything, so lets draw it. We also write the current
frames per second And the drawn primitives To the caption of the
window. The 'if (device.isWindowActive())' line is optional, but 
prevents the engine render To set the position of the mouse cursor 
after task switching when other program are active. The call To
device.yield() will avoid the busy loop To eat up all CPU
cycles when the window is Not active.
EndRem
Local lastFPS:Int = -1

While device.run()
	If device.isWindowActive()
		driver.beginScene(True, True, _SCOLOR(255,200,200,200))
		smgr.drawAll()
		driver.endScene()
	
		Local fps:Int = driver.getFPS()
	
		If (lastFPS <> fps)
			Local str:String = "Irrlicht Engine - Quake 3 Map example ["
			str :+ driver.getName()
			str :+ "] FPS:"
			str :+ fps
	
			device.setWindowCaption(str)
			lastFPS = fps
		EndIf
	Else
		device.yield()
	EndIf
Wend

Rem
In the End, Delete the Irrlicht device.
EndRem
device.drop()

Rem
That's it. Compile and play around with the program.
EndRem
