Rem* Example 006 2D Graphics

This Tutorial shows how To do 2d Graphics with the Irrlicht Engine.
It shows how To draw images, keycolor based sprites,
transparent rectangles, And different fonts. You may consider
this useful If you want To make a 2d game with the engine, Or If
you want To draw a cool interface Or head up display For your 3d game.

As always, I Include the header files, use the irr namespace,
And tell the linker To link with the .lib file.
EndRem
SuperStrict
Framework Irrlicht.Core

Rem
At first, we let the user Select the driver Type, Then start up the engine, set
a caption, And get a pointer To the video driver.
EndRem

' let user Select driver Type

Local driverType:Int=EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

' Create device

Local device:IrrlichtDevice = ..
	IrrlichtDevice.Create(driverType, _DIMENSION2DI(512, 384))

If Not device Return ' could Not create selected driver.

device.setWindowCaption("Irrlicht Engine - 2D Graphics Demo")

Local driver:IVideoDriver = device.getVideoDriver()

Rem
All 2d Graphics in this example are put together into one texture,
2ddemo.bmp. Because we want To draw colorkey based sprites, we need To
Load this texture And tell the engine, which part of it should be
transparent based on a colorkey.

In this example, we don't tell it the color directly, we just say "Hey
Irrlicht Engine, you'll find the color I want at position (0,0) on the
texture.". Instead, it would be also possible to call
driver.makeColorKeyTexture(images, _SCOLOR(0,0,0,0)), To make
e.g. all black pixels transparent. Please note that
makeColorKeyTexture just creates an alpha channel based on the color.
EndRem
Local images:ITexture = driver.getTexture("../media/2ddemo.png")
driver.makeColorKeyTextureFromPixel(images, _POSITION2DI(0,0))

Rem
To be able To draw some text with two different fonts, we first Load
them. Ok, we Load just one. As the first font we just use the Default
font which is built into the engine. Also, we define two rectangles
which specify the position of the images of the red imps (little flying
creatures) in the texture.
EndRem
Local font:IGUIFont = device.getGUIEnvironment().getBuiltInFont()
Local font2:IGUIFont = ..
	device.getGUIEnvironment().getFont("../media/fonthaettenschweiler.bmp")

Local imp1:RECTI=_RECTI(349,15,385,78)
Local imp2:RECTI=_RECTI(387,15,423,78)

Rem
Everything is prepared, now we can draw everything in the draw loop,
between the begin scene And End scene calls. In this example, we are
just doing 2d Graphics, but it would be no problem To mix them with 3d
Graphics. Just Try it out, And draw some 3d vertices Or set up a scene
with the scene manager And draw it.
EndRem
While(device.run() And driver)
	If (device.isWindowActive())

		Local time:Int = device.getTimer().getTime()

		driver.beginScene(True, True, _SCOLOR(255,120,102,136))
		
		Rem
		First, we draw 3 sprites, using the alpha channel we
		created with makeColorKeyTexture. The last parameter
		specifies that the drawing Method should use this alpha
		channel. The last-but-one parameter specifies a
		color, with which the sprite should be colored.
		(255,255,255,255) is full white, so the sprite will
		look like the original. The third sprite is drawn
		with the red channel modulated based on the time.
		EndRem

		' draw fire & dragons background world
		driver.draw2DImageSegment(images, _POSITION2DI(50,50), ..
			_RECTI(0,0,342,224), Null, ..
			_SCOLOR(255,255,255,255), True)

		Local imp:Recti
		If (time/500 Mod 2) Then imp=imp1 Else imp=imp2
		
		' draw flying imp
		driver.draw2DImageSegment(images, _POSITION2DI(164,125), ..
			 imp, Null, _SCOLOR(255,255,255,255), True)
			
		' draw second flying imp with colorcylce
		driver.draw2DImageSegment(images, _POSITION2DI(270,105), ..
			imp, Null, _SCOLOR(255,(time) Mod 255,255,255), True)

		Rem
		Drawing text is really simple. The code should be Self
		explanatory.
		EndRem

		' draw some text
		If (font) Then ..
			font.draw("This demo shows that Irrlicht is also capable of drawing 2D graphics.", ..
				_RECTI(130,10,300,50), ..
				_SCOLOR(255,255,255,255))

		' draw some other text
		If (font2) Then ..
			font2.draw("Also mixing with 3d graphics is possible.", ..
				_RECTI(130,20,300,60), ..
				_SCOLOR(255,time Mod 255,time Mod 255,255))
		

		Rem
		At last, we draw the Irrlicht Engine logo (without
		using a color Or an alpha channel) And a transparent 2d
		Rectangle at the position of the mouse cursor.
		EndRem

		' draw logo
		driver.draw2DImageSegment(images, _POSITION2DI(10,10), ..
			_RECTI(354,87,442,118))
			
		' draw transparent rect under cursor
		Local m:Position2di = device.getCursorControl().getPosition()
		driver.draw2DRectangle(_SCOLOR(100,255,255,255), ..
			_RECTI(m.getX()-20, m.getY()-20, m.getX()+20, m.getY()+20))

		driver.endScene()		
	EndIf
EndWhile

device.drop()

Rem
That's all. I hope it was not too difficult.
EndRem
