Rem
* Example 008 SpecialFX

This tutorials describes how To do special effects. It shows how To use stencil
buffer shadows, the particle system, billboards, dynamic light, And the water
surface scene node.

We start like in some tutorials before. Please note that this time, the
'shadows' flag in createDevice() is set to true, for we want to have a dynamic
shadow casted from an animated character. If this example runs too slow,
set it To False. The Irrlicht Engine checks If your hardware doesn't support
the stencil buffer, And disables shadows by itself, but just in Case the demo
runs slow on your hardware.
EndRem

SuperStrict
Framework Irrlicht.Core

Local driverType:Int=EDT_NULL

'  ask If user would like shadows

Local strInput:String=Input("Please press 'y' if you want to use realtime shadows."+Chr(13)+Chr(10)+Chr(13)+Chr(10))
Local shadows:Int=(strInput="y")

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

Rem
Create device And Exit If creation failed. We make the stencil flag
optional To avoid slow screen modes For runs without shadows.
EndRem

Local device:IrrlichtDevice = ..
	IrrlichtDevice.Create(driverType, _DIMENSION2DI(640, 480), ..
	16, False, shadows)

If Not device Return ' could Not create selected driver.

Local driver:IVideoDriver = device.getVideoDriver()
Local smgr:ISceneManager = device.getSceneManager()

Rem
For our environment, we Load a .3ds file. It is a small room I modelled
with Anim8or And exported into the 3ds format because the Irrlicht
Engine does Not support the .an8 format. I am a very bad 3d graphic
artist, And so the texture mapping is Not very nice in this model.
Luckily I am a better programmer than artist, And so the Irrlicht
Engine is able To Create a cool texture mapping For me: Just use the
mesh manipulator And Create a planar texture mapping For the mesh. If
you want To see the mapping I made with Anim8or, uncomment this line. I
also did Not figure out how To set the material Right in Anim8or, it
has a specular light color which I don't really like. I'll switch it
off too with this code.
EndRem

Local mesh:IAnimatedMesh = smgr.getMesh("../media/room.3ds")

smgr.getMeshManipulator().makePlanarTextureMapping(	mesh.getMesh(0), 0.004)

Local node:ISceneNode

node = smgr.addAnimatedMeshSceneNode(mesh)
node.setMaterialTexture(0,driver.getTexture("../media/wall.jpg"))
node.getMaterial(0).getSpecularColor().set(0,0,0,0)

Rem
Now, For the first special effect: Animated water. It works like this:
The WaterSurfaceSceneNode takes a mesh as Input And makes it wave like
a water surface. And If we let this scene node use a nice material like
the EMT_REFLECTION_2_LAYER, it looks really cool. We are doing this
with the Next few lines of code. As Input mesh, we Create a hill plane
mesh, without hills. But any other mesh could be used For this, you
could even use the room.3ds (which would look really strange) If you
want To.
EndRem

mesh = smgr.addHillPlaneMesh("myHill", ..
	_DIMENSION2DF(20,20), ..
	_DIMENSION2DI(40,40), Null, 0, ..
	_DIMENSION2DF(0,0), ..
	_DIMENSION2DF(10,10)) ..

node = smgr.addWaterSurfaceSceneNode(mesh.getMesh(0), 3.0, 300.0, 30.0)
node.setPosition(_VECTOR3DF(0,7,0))

node.setMaterialTexture(0, driver.getTexture("../media/stones.jpg"))
node.setMaterialTexture(1, driver.getTexture("../media/water.jpg"))

node.setMaterialType(EMT_REFLECTION_2_LAYER)

Rem
The second special effect is very basic, I bet you saw it already in
some Irrlicht Engine demos: A transparent billboard combined with a
dynamic light. We simply Create a light scene node, let it fly around,
And To make it look more cool, we attach a billboard scene node To it.
EndRem

' Create light

node = smgr.addLightSceneNode(Null, _VECTOR3DF(0,0,0), ..
	_SCOLORF(1.0, 0.6, 0.7, 1.0), 800.0)
Local anim:ISceneNodeAnimator
anim = smgr.createFlyCircleAnimator (_VECTOR3DF(0,150,0),250.0)
node.addAnimator(anim)
anim.drop()

' attach billboard To light

node = smgr.addBillboardSceneNode(node, _DIMENSION2DF(50, 50))
node.setMaterialFlag(EMF_LIGHTING, False)
node.setMaterialType(EMT_TRANSPARENT_ADD_COLOR)
node.setMaterialTexture(0, driver.getTexture("../media/particlewhite.bmp"))

Rem
The Next special effect is a lot more interesting: A particle system.
The particle system in the Irrlicht Engine is quite modular And
extensible, but yet easy To use. There is a particle system scene node
into which you can put a particle emitter, which makes particles come out
of nothing. These emitters are quite flexible And usually have lots of
parameters like direction, amount, And color of the particles they
Create.

There are different emitters, For example a point emitter which lets
particles pop out at a fixed point. If the particle emitters available
in the engine are Not enough For you, you can easily Create your own
ones, you'll simply have to create a class derived from the
IParticleEmitter interface And attach it To the particle system using
setEmitter(). In this example we Create a box particle emitter, which
creates particles randomly inside a box. The parameters define the box,
direction of the particles, minimal And maximal New particles per
second, color, And minimal And maximal lifetime of the particles.

Because only with emitters particle system would be a little bit
boring, there are particle affectors which modify particles While
they fly around. Affectors can be added To a particle system For
simulating additional effects like gravity Or wind.
The particle affector we use in this example is an affector which
modifies the color of the particles: It lets them fade out. Like the
particle emitters, additional particle affectors can also be
implemented by you, simply derive a class from IParticleAffector And
add it with addAffector().

After we set a nice material To the particle system, we have a cool
looking camp fire. By adjusting material, texture, particle emitter,
And affector parameters, it is also easily possible To Create smoke,
rain, explosions, snow, And so on.
EndRem

' Create a particle system

Local ps:IParticleSystemSceneNode = ..
	smgr.addParticleSystemSceneNode(False)

Local em:IParticleEmitter = ps.createBoxEmitter( ..
	AABBox3df.CreateFromVals(-7,0,-7,7,1,7), .. ' emitter size
	_VECTOR3DF(0.0,0.06,0.0), ..  ' initial direction
	80,100, ..                    ' emit rate
	_SCOLOR(0,255,255,255), ..    ' darkest color
	_SCOLOR(0,255,255,255), ..    ' brightest color
	800,2000,0, ..                ' Min And Max age, angle
	_DIMENSION2DF(10,10), ..      ' Min size
	_DIMENSION2DF(20,20))         ' Max size

ps.setEmitter(em) ' this grabs the emitter
em.drop() ' so we can drop it here without deleting it

Local paf:IParticleAffector = ps.createFadeOutParticleAffector()

ps.addAffector(paf) ' same goes For the affector
paf.drop()

ps.setPosition(_VECTOR3DF(-70,60,40))
ps.SetScale(_VECTOR3DF(2,2,2))
ps.setMaterialFlag(EMF_LIGHTING, False)
ps.setMaterialFlag(EMF_ZWRITE_ENABLE, False)
ps.setMaterialTexture(0, driver.getTexture("../media/fire.bmp"))
ps.setMaterialType(EMT_TRANSPARENT_VERTEX_ALPHA)

Rem
Next we add a volumetric light node, which adds a glowing fake area light To
the scene. Like with the billboards And particle systems we also assign a
texture For the desired effect, though this time we'll use a texture animator
To Create the illusion of a magical glowing area effect.
EndRem
Local n:IVolumeLightSceneNode = smgr.addVolumeLightSceneNode(Null, -1, ..
			32, ..                         	' Subdivisions on U axis
			32, ..                         	' Subdivisions on V axis
			_SCOLOR(0, 255, 255, 255), .. 	' foot color
			_SCOLOR(0, 0, 0, 0))      		' tail color

If (n)

	n.SetScale(_VECTOR3DF(56.0, 56.0, 56.0))
	n.setPosition(_VECTOR3DF(-120,50,40))

	' Load textures For animation
	Local textures:ITexture[7]
	
	For Local g:Int = 7 Until 0 Step -1	
		Local tmp:String 
		tmp = "../media/portal"
		tmp :+ g
		tmp :+ ".bmp"
		Local t:ITexture = driver.getTexture( tmp )
		textures[7 - g] = t
	Next

	' Create texture animator
	Local glow:ISceneNodeAnimator = smgr.createTextureAnimator(textures, 150)

	' add the animator
	n.addAnimator(glow)

	' drop the animator because it was created with a Create() Function
	glow.drop()
EndIf

Rem
As our last special effect, we want a dynamic shadow be casted from an
animated character. For this we Load a DirectX .x model And place it
into our world. For creating the shadow, we simply need To call
addShadowVolumeSceneNode(). The color of shadows is only adjustable
globally For all shadows, by calling ISceneManager::setShadowColor().
Voila, here is our dynamic shadow.

Because the character is a little bit too small For this scene, we make
it bigger using SetScale(). And because the character is lighted by a
dynamic light, we need To normalize the normals To make the lighting on
it correct. This is always necessary If the scale of a dynamic lighted
model is Not (1,1,1). Otherwise it would get too dark Or too bright
because the normals will be scaled too.
EndRem

' add animated character

mesh = smgr.getMesh("../media/dwarf.x")
Local anode:IAnimatedMeshSceneNode

anode = smgr.addAnimatedMeshSceneNode(mesh)
anode.setPosition(_VECTOR3DF(-50,20,-60))
anode.setAnimationSpeed(15)

' add shadow
anode.addShadowVolumeSceneNode()
smgr.setShadowColor(_SCOLOR(150,0,0,0))

' make the model a little bit bigger And normalize its normals
' because of the scaling, For correct lighting
anode.SetScale(_VECTOR3DF(2,2,2))
anode.setMaterialFlag(EMF_NORMALIZE_NORMALS, True)

Rem
Finally we simply have To draw everything, that's all.
EndRem

Local camera:ICameraSceneNode = smgr.addCameraSceneNodeFPS()
camera.setPosition(_VECTOR3DF(-50,50,-150))

' disable mouse cursor
device.getCursorControl().setVisible(False)

Local lastFPS:Int = -1

While(device.run())
	If (device.isWindowActive())
		driver.beginScene(True, True, Null)
	
		smgr.drawAll()
	
		driver.endScene()
	
		Local fps:Int = driver.getFPS()
	
		If (lastFPS <> fps)
		  Local str:String = "Irrlicht Engine - SpecialFX example ["
		  str :+ driver.getName()
		  str :+ "] FPS:"
		  str :+ fps
	
		  device.setWindowCaption(str)
		  lastFPS = fps
		EndIf
	EndIf
EndWhile

device.drop()
