Rem
* Example 013 Render To Texture

This tutorial shows how To render To a texture using Irrlicht. Render To
texture is a feature with which it is possible To Create nice special effects.
In addition, this tutorial shows how To enable specular highlights.

In the beginning, everything as usual. Include the needed headers, ask the user
For the rendering driver, Create the Irrlicht Device:
EndRem

SuperStrict
Framework Irrlicht.Core

Local driverType:Int=EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

' create device And Exit If creation failed

Local device:IrrlichtDevice = ..
	IrrlichtDevice.Create(driverType, _DIMENSION2DI(640, 480),16,False,False)

If Not device Return ' could Not create selected driver.

Local driver:IVideoDriver = device.getVideoDriver()
Local smgr:ISceneManager = device.getSceneManager()
Local env:IGUIEnvironment = device.getGUIEnvironment()

Rem
Now, we Load an animated mesh To be displayed. As in most examples,
we'll take the fairy md2 model. The difference here: We set the
shininess of the model To a value other than 0 which is the Default
value. This enables specular highlights on the model If dynamic
lighting is on. The value influences the size of the highlights.
EndRem

' Load And display animated fairy mesh

Local fairy:IAnimatedMeshSceneNode = smgr.addAnimatedMeshSceneNode( ..
	smgr.getMesh("../media/faerie.md2"))

If (fairy)
	fairy.setMaterialTexture(0, driver.getTexture("../media/faerie2.bmp")) '  set diffuse texture
	fairy.setMaterialFlag(EMF_LIGHTING, True) '  enable dynamic lighting
	fairy.getMaterial(0).setShininess(20.0) '  set size of specular highlights
	fairy.setPosition(_VECTOR3DF(-10,0,-100))
	fairy.setMD2Animation( EMAT_STAND )
EndIf

Rem
To make specular highlights appear on the model, we need a dynamic
light in the scene. We add one directly in vicinity of the model. In
addition, To make the model Not that dark, we set the ambient light To
gray.
EndRem

' add white light
Local light:ILightSceneNode = smgr.addLightSceneNode(Null, ..
	_VECTOR3DF(-15,5,-105), _SCOLORF(1.0, 1.0, 1.0))

'  set ambient light
smgr.setAmbientLight(SColorf.createFromSColor(_SCOLOR(0,60,60,60)))

Rem
The Next is just some standard stuff: Add a user controlled camera To
the scene, disable mouse cursor, And add a test cube And let it rotate
To make the scene more interesting.
EndRem

' add fps camera
Local fpsCamera:ICameraSceneNode = smgr.addCameraSceneNodeFPS()
fpsCamera.setPosition(_VECTOR3DF(-50,50,-150))

' disable mouse cursor
device.getCursorControl().setVisible(False)

' Create test cube
Local test:ISceneNode = smgr.addCubeSceneNode(60)

' let the cube rotate And set some light settings
Local anim:ISceneNodeAnimator = smgr.createRotationAnimator( ..
	_VECTOR3DF(0.3, 0.3,0))

test.setPosition(_VECTOR3DF(-100,0,-100))
test.setMaterialFlag(EMF_LIGHTING, False) '  disable dynamic lighting
test.addAnimator(anim)
anim.drop()

' set window caption
device.setWindowCaption("Irrlicht Engine - Render to Texture and Specular Highlights example")

Rem
To test out the render To texture feature, we need a render target
texture. These are Not like standard textures, but need To be created
first. To Create one, we call IVideoDriver::addRenderTargetTexture()
And specify the size of the texture. Please don't use sizes bigger than
the frame buffer For this, because the render target shares the zbuffer
with the frame buffer.
Because we want To render the scene Not from the user camera into the
texture, we add another fixed camera To the scene. But before we do all
this, we check If the current running driver is able To render To
textures. If it is Not, we simply display a warning text.
EndRem

' Create render target
Local rt:ITexture
Local fixedCam:ICameraSceneNode

If (driver.queryFeature(EVDF_RENDER_TO_TARGET))

	rt = driver.addRenderTargetTexture(_DIMENSION2DI(256,256), "RTT1")
	test.setMaterialTexture(0, rt) '  set material of cube To render target

	' add fixed camera
	fixedCam = smgr.addCameraSceneNode(Null, _VECTOR3DF(10,10,-80), ..
		_VECTOR3DF(-10,10,-100))
Else
	' create problem text
	Local skin:IGUISkin = env.getSkin()
	Local font:IGUIFont = env.getFont("../media/fonthaettenschweiler.bmp")
	If (font) Then ..
		skin.setFont(font)

	Local text:IGUIStaticText = env.addStaticText( ..
		"Your hardware or this renderer is not able to use the "+ ..
		"render to texture feature. RTT Disabled.", ..
		_RECTI(150,20,470,60))

	text.setOverrideColor(_SCOLOR(100,255,255,255))
EndIf

Rem
Nearly finished. Now we need To draw everything. Every frame, we draw
the scene twice. Once from the fixed camera into the render target
texture And once as usual. When rendering into the render target, we
need To disable the visibilty of the test cube, because it has the
render target texture applied To it. That's it, wasn't too complicated
I hope. :)
EndRem

Local lastFPS:Int = -1

While(device.run())
	If (device.isWindowActive())
		driver.beginScene(True, True, Null)
	
		If (rt)
			'  draw scene into render target
			
			'  set render target texture
			driver.setRenderTarget(rt, True, True, _SCOLOR(0,0,0,255))     
	
			'  make cube invisible And set fixed camera as active camera
			test.setVisible(False)
			smgr.setActiveCamera(fixedCam)
	
			'  draw whole scene into render buffer
			smgr.drawAll()                 
	
			'  set back old render target
			driver.setRenderTarget(Null, True, True, Null)      
	
			'  make the cube visible And set the user controlled camera as active one
			test.setVisible(True)
			smgr.setActiveCamera(fpsCamera)
		EndIf

		'  draw scene normally
		smgr.drawAll() 
		env.drawAll()
	
		driver.endScene()
	
		'  display frames per second in window title
		Local fps:Int = driver.getFPS()
		If (lastFPS <> fps)
			Local str:String = "Irrlicht Engine - Render to Texture and Specular Highlights example"
			str :+ " FPS:"
			str :+ fps
	
			device.setWindowCaption(str)
			lastFPS = fps
		EndIf
	EndIf
Wend

device.drop() ' drop device

